-------------------------------------------------------------
-- Daan Leijen 1997,  leijen@fwi.uva.nl
--
-- There is a paper at the haskell script home page now
-- which explains more about these combinators
-------------------------------------------------------------

module AgentScript ( module Automation, module AgentServer, 
                    module AgentScript, module Notify ) where

import Automation
import List
import AgentServer 
import Notify

infixl 5 <*>
infixl 4 <|>


forall :: (IUnknown a -> IO b) -> [IUnknown a] -> IO ()
forall action objects = sequence  (map (#action) objects)


-----------------------------------------------------------
-- IAgent utils
-----------------------------------------------------------

moveToPos (x,y) v  = moveTo x y v

speedInstantly  = 0
speedFast       = 1500
speedNormal     = 3000

instantly       = 1
normally        = 0

type ReqID      = Int
type CharID     = Int


slowShow        = showUp normally
slowHide        = hide normally
fastShow        = showUp instantly
fastHide        = hide instantly

loadCharacter :: String -> IAgent a -> IO (IAgentCharacter Interface)
loadCharacter name server       
                = do (charid,reqid) <- server # load filename
                     server # getCharacter charid
                where
                  filename  = "\\Program Files\\Microsoft Agent\\Characters\\"
                                ++ name ++ ".acs"


unloadCharacter :: IAgentCharacter b -> IAgent a -> IO ()
unloadCharacter agent server
                = do reqid <- agent # slowHide
                     server # waitForCompletion reqid

-------------------------------------------------------------
-- Sequential and Parallel composition
-------------------------------------------------------------


type Action   = (IAgentCharacter Interface) -> IO ReqID
type Anim       = ([(IAgentCharacter Interface)],IO ReqID)


(<*>) :: Anim -> Anim -> Anim
(agents1,action1) <*> (agents2,action2)    
        = (agents1 `union` agents2,
           do reqid1 <- action1
              (agents2 \\ agents1) # waitfor reqid1
              reqid2 <- action2
              (agents1 \\ agents2) # waitfor reqid2 
         )

(<|>) :: Anim -> Anim -> Anim
(agents1,action1) <|> (agents2,action2)    
        = (agents1 `union` agents2,
           do reqid1 <- action1
              reqid2 <- action2
              (agents2 \\ agents1) # waitfor reqid1
              (agents1 \\ agents2) # waitfor reqid2 
         )

waitfor :: ReqID -> [(IAgentCharacter c)] -> IO ReqID
waitfor reqid agents = do agents # forall (wait reqid)
                          return reqid                      


while :: Anim -> Anim -> Anim
while ([agent1],action1) (agents2,action2)
        = ([agent1] `union` agents2,
           do stopid <- action1
              reqid  <- action2
              (agents2 \\ [agent1]) # forall (interrupt stopid) 
              return reqid 
         )


-------------------------------------------------------------
-- Run the animations
-------------------------------------------------------------

animate1 :: (IAgentCharacter Interface) -> Action -> Anim
animate1 agent action
        = ([agent], agent # action)

animate :: (IAgentCharacter Interface) -> [Action] -> Anim
animate agent actions           
        = ([agent], doactions actions)
        where
          doactions []     = error "AgentInterfacebinators.animate: empty animation list"
          doactions [x]    = agent # x
          doactions (x:xs) = do agent # x
                                doactions xs

runAnim (agents,actions) server
        = do sink       <- coCreateInstance clsidHaskellNotifySink interfaceNULL 
                                             LocalProcess 
                                             iidIDispatch
             sinkID     <- server # register sink
             reqid      <- actions
             sink # completeRequest reqid
             server # unregister sinkID
             return ()

waitForCompletion reqid server
        = do sink       <- coCreateInstance clsidHaskellNotifySink interfaceNULL 
                                             LocalProcess 
                                             iidIDispatch
             sinkID     <- server # register sink
             sink # completeRequest reqid
             server # unregister sinkID
             return ()

seqAnim = foldr1 (<*>)



-----------------------------------------------------------
-- Tricky things
--
-- @ioWhen@ breaks the abstraction of the combinators. It
-- is also implemtented a bit tricky. We should use the @IAgentNotifySink@
-- to implement IO actions, but this will do for now.
-----------------------------------------------------------

ioWhen :: ((IAgentCharacter Interface) -> IO Bool) -> (IAgentCharacter Interface) -> IO a -> Anim
ioWhen test agent io   = ([agent],
                           do waitloop
                              io
                              vis <- agent # getVisible 
                              if (vis==0) then agent # hide instantly
                                          else agent # showUp instantly
                           )
                        where
                          waitloop = do b <- agent # test 
                                        if (b) then return ()
                                               else waitloop



completes agent
              = do agent # moveToPos endPos speedInstantly
                   waitloop
                   agent # hide instantly
            where
              endPos     = (2999,2999)
              waitloop   = do pos <- agent # getPosition
                              if (pos == endPos)
                               then return 0
                               else waitloop 
                               

