module AgentServer where

import qualified Prelude
import qualified Automation (IDispatch, Interface, Variant, 
                             outInt, inVariant, method2, inInt, method0, inIUnknown, method1, 
                             outIDispatch, outString, outVariant, method3, inString, 
                             outIUnknown)
import qualified Com (CLSID, mkCLSID, IID, mkIID, IUnknown)

-- --------------------------------------------------
-- 
-- coclass AgentServer
-- 
-- --------------------------------------------------
clsidAgentServer :: Com.CLSID
clsidAgentServer =
  Com.mkCLSID "{A7B93C92-7B81-11D0-AC5F-00C04FD97575}"
-- --------------------------------------------------
-- 
-- interface IAgent
-- 
-- --------------------------------------------------
data Agent a = Agent
                
type IAgent a = Automation.IDispatch (Agent a)
iidIAgent :: Com.IID (IAgent Automation.Interface)
iidIAgent = Com.mkIID "{A7B93C91-7B81-11D0-AC5F-00C04FD97575}"
load :: (Automation.Variant a0) => a0 -> IAgent a -> Prelude.IO (Prelude.Int, Prelude.Int)
load vLoadKey =
  Automation.method2 "Load" [Automation.inVariant vLoadKey] Automation.outInt Automation.outInt
unload :: Prelude.Int -> IAgent a -> Prelude.IO ()
unload dwCharID =
  Automation.method0 "Unload" [Automation.inInt dwCharID]
register :: IAgentNotifySink a0 -> IAgent a -> Prelude.IO (Prelude.Int)
register punkNotifySink =
  Automation.method1 "Register" [Automation.inIUnknown punkNotifySink] Automation.outInt
unregister :: Prelude.Int -> IAgent a -> Prelude.IO ()
unregister dwSinkID =
  Automation.method0 "Unregister" [Automation.inInt dwSinkID]
getCharacter :: Prelude.Int -> IAgent a -> Prelude.IO (IAgentCharacter Com.Interface)
getCharacter dwCharID =
  Automation.method1 "GetCharacter" [Automation.inInt dwCharID] Automation.outIDispatch
getSuspended :: IAgent a -> Prelude.IO (Prelude.Int)
getSuspended =
  Automation.method1 "GetSuspended" [] Automation.outInt
-- --------------------------------------------------
-- 
-- interface IAgentUserInput
-- 
-- --------------------------------------------------
data AgentUserInput a = AgentUserInput
                         
type IAgentUserInput a = Automation.IDispatch (AgentUserInput a)
iidIAgentUserInput :: Com.IID (IAgentUserInput Automation.Interface)
iidIAgentUserInput =
  Com.mkIID "{A7B93C80-7B81-11D0-AC5F-00C04FD97575}"
getCount :: Automation.IDispatch a -> Prelude.IO (Prelude.Int)
getCount = Automation.method1 "GetCount" [] Automation.outInt
getItemID :: Prelude.Int -> IAgentUserInput a -> Prelude.IO (Prelude.Int)
getItemID dwItemIndex =
  Automation.method1 "GetItemID" [Automation.inInt dwItemIndex] Automation.outInt
getItemConfidence :: Prelude.Int -> IAgentUserInput a -> Prelude.IO (Prelude.Int)
getItemConfidence dwItemIndex =
  Automation.method1 "GetItemConfidence" [Automation.inInt dwItemIndex] Automation.outInt
getItemText :: Prelude.Int -> IAgentUserInput a -> Prelude.IO (Prelude.String)
getItemText dwItemIndex =
  Automation.method1 "GetItemText" [Automation.inInt dwItemIndex] Automation.outString
getAllItemData :: (Automation.Variant a0, Automation.Variant a1, Automation.Variant a2) => IAgentUserInput a -> Prelude.IO (a0, a1, a2)
getAllItemData =
  Automation.method3 "GetAllItemData" [] Automation.outVariant Automation.outVariant Automation.outVariant
-- --------------------------------------------------
-- 
-- interface IAgentCommand
-- 
-- --------------------------------------------------
data AgentCommand a = AgentCommand
                       
type IAgentCommand a = Automation.IDispatch (AgentCommand a)
iidIAgentCommand :: Com.IID (IAgentCommand Automation.Interface)
iidIAgentCommand =
  Com.mkIID "{A7B93C83-7B81-11D0-AC5F-00C04FD97575}"
setCaption :: Prelude.String -> Automation.IDispatch a -> Prelude.IO ()
setCaption bszCaption =
  Automation.method0 "SetCaption" [Automation.inString bszCaption]
getCaption :: Automation.IDispatch a -> Prelude.IO (Prelude.String)
getCaption =
  Automation.method1 "GetCaption" [] Automation.outString
setVoice :: Prelude.String -> Automation.IDispatch a -> Prelude.IO ()
setVoice bszVoice =
  Automation.method0 "SetVoice" [Automation.inString bszVoice]
getVoice :: Automation.IDispatch a -> Prelude.IO (Prelude.String)
getVoice = Automation.method1 "GetVoice" [] Automation.outString
setEnabled :: Prelude.Int -> IAgentCommand a -> Prelude.IO ()
setEnabled bEnabled =
  Automation.method0 "SetEnabled" [Automation.inInt bEnabled]
getEnabled :: Automation.IDispatch a -> Prelude.IO (Prelude.Int)
getEnabled = Automation.method1 "GetEnabled" [] Automation.outInt
setVisible :: Prelude.Int -> Automation.IDispatch a -> Prelude.IO ()
setVisible bVisible =
  Automation.method0 "SetVisible" [Automation.inInt bVisible]
getVisible :: Automation.IDispatch a -> Prelude.IO (Prelude.Int)
getVisible = Automation.method1 "GetVisible" [] Automation.outInt
setConfidenceThreshold :: Prelude.Int -> IAgentCommand a -> Prelude.IO ()
setConfidenceThreshold lThreshold =
  Automation.method0 "SetConfidenceThreshold" [Automation.inInt lThreshold]
getConfidenceThreshold :: IAgentCommand a -> Prelude.IO (Prelude.Int)
getConfidenceThreshold =
  Automation.method1 "GetConfidenceThreshold" [] Automation.outInt
setConfidenceText :: Prelude.String -> IAgentCommand a -> Prelude.IO ()
setConfidenceText bszTipText =
  Automation.method0 "SetConfidenceText" [Automation.inString bszTipText]
getConfidenceText :: IAgentCommand a -> Prelude.IO (Prelude.String)
getConfidenceText =
  Automation.method1 "GetConfidenceText" [] Automation.outString
getID :: IAgentCommand a -> Prelude.IO (Prelude.Int)
getID = Automation.method1 "GetID" [] Automation.outInt
-- --------------------------------------------------
-- 
-- interface IAgentCommands
-- 
-- --------------------------------------------------
data AgentCommands a = AgentCommands
                        
type IAgentCommands a = Automation.IDispatch (AgentCommands a)
iidIAgentCommands :: Com.IID (IAgentCommands Automation.Interface)
iidIAgentCommands =
  Com.mkIID "{A7B93C85-7B81-11D0-AC5F-00C04FD97575}"
getCommand :: Prelude.Int -> IAgentCommands a -> Prelude.IO (Com.IUnknown Automation.Interface)
getCommand dwCommandID =
  Automation.method1 "GetCommand" [Automation.inInt dwCommandID] Automation.outIUnknown
add :: Prelude.String -> Prelude.String -> Prelude.Int -> Prelude.Int -> IAgentCommands a -> Prelude.IO (Prelude.Int)
add bszCaption bszVoice bEnabled bVisible =
  Automation.method1 "Add" [Automation.inString bszCaption, Automation.inString bszVoice, Automation.inInt bEnabled, Automation.inInt bVisible] Automation.outInt
insert :: Prelude.String -> Prelude.String -> Prelude.Int -> Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentCommands a -> Prelude.IO (Prelude.Int)
insert bszCaption bszVoice bEnabled bVisible dwRefID bBefore =
  Automation.method1 "Insert" [Automation.inString bszCaption, Automation.inString bszVoice, Automation.inInt bEnabled, Automation.inInt bVisible, Automation.inInt dwRefID, Automation.inInt bBefore] Automation.outInt
remove :: Prelude.Int -> IAgentCommands a -> Prelude.IO ()
remove dwID = Automation.method0 "Remove" [Automation.inInt dwID]
removeAll :: IAgentCommands a -> Prelude.IO ()
removeAll = Automation.method0 "RemoveAll" []
-- --------------------------------------------------
-- 
-- interface IAgentSpeechInputProperties
-- 
-- --------------------------------------------------
data AgentSpeechInputProperties a = AgentSpeechInputProperties
                                     
type IAgentSpeechInputProperties a = Automation.IDispatch (AgentSpeechInputProperties a)
iidIAgentSpeechInputProperties :: Com.IID (IAgentSpeechInputProperties Automation.Interface)
iidIAgentSpeechInputProperties =
  Com.mkIID "{A7B93C87-7B81-11D0-AC5F-00C04FD97575}"
getInstalled :: IAgentSpeechInputProperties a -> Prelude.IO (Prelude.Int)
getInstalled =
  Automation.method1 "GetInstalled" [] Automation.outInt
getHotKey :: IAgentSpeechInputProperties a -> Prelude.IO (Prelude.String)
getHotKey = Automation.method1 "GetHotKey" [] Automation.outString
getLCID :: IAgentSpeechInputProperties a -> Prelude.IO (Prelude.Int)
getLCID = Automation.method1 "GetLCID" [] Automation.outInt
getEngine :: IAgentSpeechInputProperties a -> Prelude.IO (Prelude.String)
getEngine = Automation.method1 "GetEngine" [] Automation.outString
setEngine :: Prelude.String -> IAgentSpeechInputProperties a -> Prelude.IO ()
setEngine bszEngine =
  Automation.method0 "SetEngine" [Automation.inString bszEngine]
getListeningTip :: IAgentSpeechInputProperties a -> Prelude.IO (Prelude.Int)
getListeningTip =
  Automation.method1 "GetListeningTip" [] Automation.outInt
-- --------------------------------------------------
-- 
-- interface IAgentAudioOutputProperties
-- 
-- --------------------------------------------------
data AgentAudioOutputProperties a = AgentAudioOutputProperties
                                     
type IAgentAudioOutputProperties a = Automation.IDispatch (AgentAudioOutputProperties a)
iidIAgentAudioOutputProperties :: Com.IID (IAgentAudioOutputProperties Automation.Interface)
iidIAgentAudioOutputProperties =
  Com.mkIID "{A7B93C89-7B81-11D0-AC5F-00C04FD97575}"
getUsingSoundEffects :: IAgentAudioOutputProperties a -> Prelude.IO (Prelude.Int)
getUsingSoundEffects =
  Automation.method1 "GetUsingSoundEffects" [] Automation.outInt
-- --------------------------------------------------
-- 
-- interface IAgentPropertySheet
-- 
-- --------------------------------------------------
data AgentPropertySheet a = AgentPropertySheet
                             
type IAgentPropertySheet a = Automation.IDispatch (AgentPropertySheet a)
iidIAgentPropertySheet :: Com.IID (IAgentPropertySheet Automation.Interface)
iidIAgentPropertySheet =
  Com.mkIID "{A7B93C8B-7B81-11D0-AC5F-00C04FD97575}"
getPosition :: Automation.IDispatch a -> Prelude.IO (Prelude.Int, Prelude.Int)
getPosition =
  Automation.method2 "GetPosition" [] Automation.outInt Automation.outInt
getSize :: Automation.IDispatch a -> Prelude.IO (Prelude.Int, Prelude.Int)
getSize =
  Automation.method2 "GetSize" [] Automation.outInt Automation.outInt
getPage :: IAgentPropertySheet a -> Prelude.IO (Prelude.String)
getPage = Automation.method1 "GetPage" [] Automation.outString
setPage :: Prelude.String -> IAgentPropertySheet a -> Prelude.IO ()
setPage bszPage =
  Automation.method0 "SetPage" [Automation.inString bszPage]
-- --------------------------------------------------
-- 
-- interface IAgentBalloon
-- 
-- --------------------------------------------------
data AgentBalloon a = AgentBalloon
                       
type IAgentBalloon a = Automation.IDispatch (AgentBalloon a)
iidIAgentBalloon :: Com.IID (IAgentBalloon Automation.Interface)
iidIAgentBalloon =
  Com.mkIID "{A7B93C8D-7B81-11D0-AC5F-00C04FD97575}"
getNumLines :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getNumLines = Automation.method1 "GetNumLines" [] Automation.outInt
getNumCharsPerLine :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getNumCharsPerLine =
  Automation.method1 "GetNumCharsPerLine" [] Automation.outInt
getFontName :: IAgentBalloon a -> Prelude.IO (Prelude.String)
getFontName =
  Automation.method1 "GetFontName" [] Automation.outString
getFontSize :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getFontSize = Automation.method1 "GetFontSize" [] Automation.outInt
getFontBold :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getFontBold = Automation.method1 "GetFontBold" [] Automation.outInt
getFontItalic :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getFontItalic =
  Automation.method1 "GetFontItalic" [] Automation.outInt
getFontStrikethru :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getFontStrikethru =
  Automation.method1 "GetFontStrikethru" [] Automation.outInt
getFontUnderline :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getFontUnderline =
  Automation.method1 "GetFontUnderline" [] Automation.outInt
getForeColor :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getForeColor =
  Automation.method1 "GetForeColor" [] Automation.outInt
getBackColor :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getBackColor =
  Automation.method1 "GetBackColor" [] Automation.outInt
getBorderColor :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getBorderColor =
  Automation.method1 "GetBorderColor" [] Automation.outInt
setFontName :: Prelude.String -> IAgentBalloon a -> Prelude.IO ()
setFontName bszFontName =
  Automation.method0 "SetFontName" [Automation.inString bszFontName]
setFontSize :: Prelude.Int -> IAgentBalloon a -> Prelude.IO ()
setFontSize lFontSize =
  Automation.method0 "SetFontSize" [Automation.inInt lFontSize]
setFontCharSet :: Prelude.Int -> IAgentBalloon a -> Prelude.IO ()
setFontCharSet sFontCharSet =
  Automation.method0 "SetFontCharSet" [Automation.inInt sFontCharSet]
getFontCharSet :: IAgentBalloon a -> Prelude.IO (Prelude.Int)
getFontCharSet =
  Automation.method1 "GetFontCharSet" [] Automation.outInt
-- --------------------------------------------------
-- 
-- interface IAgentCharacter
-- 
-- --------------------------------------------------
data AgentCharacter a = AgentCharacter
                         
type IAgentCharacter a = Automation.IDispatch (AgentCharacter a)
iidIAgentCharacter :: Com.IID (IAgentCharacter Automation.Interface)
iidIAgentCharacter =
  Com.mkIID "{A7B93C8F-7B81-11D0-AC5F-00C04FD97575}"
setPosition :: Prelude.Int -> Prelude.Int -> IAgentCharacter a -> Prelude.IO ()
setPosition lLeft lTop =
  Automation.method0 "SetPosition" [Automation.inInt lLeft, Automation.inInt lTop]
setSize :: Prelude.Int -> Prelude.Int -> IAgentCharacter a -> Prelude.IO ()
setSize lWidth lHeight =
  Automation.method0 "SetSize" [Automation.inInt lWidth, Automation.inInt lHeight]
getName :: IAgentCharacter a -> Prelude.IO (Prelude.String)
getName = Automation.method1 "GetName" [] Automation.outString
getDescription :: IAgentCharacter a -> Prelude.IO (Prelude.String)
getDescription =
  Automation.method1 "GetDescription" [] Automation.outString
getTTSSpeed :: IAgentCharacter a -> Prelude.IO (Prelude.Int)
getTTSSpeed = Automation.method1 "GetTTSSpeed" [] Automation.outInt
getTTSPitch :: IAgentCharacter a -> Prelude.IO (Prelude.Int)
getTTSPitch = Automation.method1 "GetTTSPitch" [] Automation.outInt
activate :: Prelude.Int -> IAgentCharacter a -> Prelude.IO ()
activate sState =
  Automation.method0 "Activate" [Automation.inInt sState]
setIdleOn :: Prelude.Int -> IAgentCharacter a -> Prelude.IO ()
setIdleOn bOn =
  Automation.method0 "SetIdleOn" [Automation.inInt bOn]
getIdleOn :: IAgentCharacter a -> Prelude.IO (Prelude.Int)
getIdleOn = Automation.method1 "GetIdleOn" [] Automation.outInt
prepare :: Prelude.Int -> Prelude.String -> Prelude.Int -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
prepare dwType bszName bQueue =
  Automation.method1 "Prepare" [Automation.inInt dwType, Automation.inString bszName, Automation.inInt bQueue] Automation.outInt
play :: Prelude.String -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
play bszAnimation =
  Automation.method1 "Play" [Automation.inString bszAnimation] Automation.outInt
stop :: Prelude.Int -> IAgentCharacter a -> Prelude.IO ()
stop dwReqID = Automation.method0 "Stop" [Automation.inInt dwReqID]
stopAll :: Prelude.Int -> IAgentCharacter a -> Prelude.IO ()
stopAll lTypes =
  Automation.method0 "StopAll" [Automation.inInt lTypes]
wait :: Prelude.Int -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
wait dwReqID =
  Automation.method1 "Wait" [Automation.inInt dwReqID] Automation.outInt
interrupt :: Prelude.Int -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
interrupt dwReqID =
  Automation.method1 "Interrupt" [Automation.inInt dwReqID] Automation.outInt
showUp :: Prelude.Int -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
showUp bFast =
  Automation.method1 "Show" [Automation.inInt bFast] Automation.outInt
hide :: Prelude.Int -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
hide bFast =
  Automation.method1 "Hide" [Automation.inInt bFast] Automation.outInt
speak s  = speak' s ""
speak' :: Prelude.String -> Prelude.String -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
speak' bszText bszUrl =
  Automation.method1 "Speak" [Automation.inString bszText, Automation.inString bszUrl] Automation.outInt
moveTo :: Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
moveTo x y lSpeed =
  Automation.method1 "MoveTo" [Automation.inInt x, Automation.inInt y, Automation.inInt lSpeed] Automation.outInt
gestureAt :: Prelude.Int -> Prelude.Int -> IAgentCharacter a -> Prelude.IO (Prelude.Int)
gestureAt x y =
  Automation.method1 "GestureAt" [Automation.inInt x, Automation.inInt y] Automation.outInt
getMoveCause :: IAgentCharacter a -> Prelude.IO (Prelude.Int)
getMoveCause =
  Automation.method1 "GetMoveCause" [] Automation.outInt
getVisibilityCause :: IAgentCharacter a -> Prelude.IO (Prelude.Int)
getVisibilityCause =
  Automation.method1 "GetVisibilityCause" [] Automation.outInt
hasOtherClients :: IAgentCharacter a -> Prelude.IO (Prelude.Int)
hasOtherClients =
  Automation.method1 "HasOtherClients" [] Automation.outInt
setSoundEffectsOn :: Prelude.Int -> IAgentCharacter a -> Prelude.IO ()
setSoundEffectsOn bOn =
  Automation.method0 "SetSoundEffectsOn" [Automation.inInt bOn]
getSoundEffectsOn :: IAgentCharacter a -> Prelude.IO (Prelude.Int)
getSoundEffectsOn =
  Automation.method1 "GetSoundEffectsOn" [] Automation.outInt
setName :: Prelude.String -> IAgentCharacter a -> Prelude.IO ()
setName bszName =
  Automation.method0 "SetName" [Automation.inString bszName]
setDescription :: Prelude.String -> IAgentCharacter a -> Prelude.IO ()
setDescription bszDescription =
  Automation.method0 "SetDescription" [Automation.inString bszDescription]
getExtraData :: IAgentCharacter a -> Prelude.IO (Prelude.String)
getExtraData =
  Automation.method1 "GetExtraData" [] Automation.outString
-- --------------------------------------------------
-- 
-- interface IAgentNotifySink
-- 
-- --------------------------------------------------
data AgentNotifySink a = AgentNotifySink
                          
type IAgentNotifySink a = Automation.IDispatch (AgentNotifySink a)
iidIAgentNotifySink :: Com.IID (IAgentNotifySink Automation.Interface)
iidIAgentNotifySink =
  Com.mkIID "{00D18159-8466-11D0-AC63-00C04FD97575}"
command :: Prelude.Int -> Com.IUnknown a0 -> IAgentNotifySink a -> Prelude.IO ()
command dwCommandID punkUserInput =
  Automation.method0 "Command" [Automation.inInt dwCommandID, Automation.inIUnknown punkUserInput]
activateInputState :: Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
activateInputState dwCharID bActivated =
  Automation.method0 "ActivateInputState" [Automation.inInt dwCharID, Automation.inInt bActivated]
restart :: IAgentNotifySink a -> Prelude.IO ()
restart = Automation.method0 "Restart" []
shutdown :: IAgentNotifySink a -> Prelude.IO ()
shutdown = Automation.method0 "Shutdown" []
visibleState :: Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
visibleState dwCharID bVisible dwCause =
  Automation.method0 "VisibleState" [Automation.inInt dwCharID, Automation.inInt bVisible, Automation.inInt dwCause]
click :: Prelude.Int -> Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
click dwCharID fwKeys x y =
  Automation.method0 "Click" [Automation.inInt dwCharID, Automation.inInt fwKeys, Automation.inInt x, Automation.inInt y]
dblClick :: Prelude.Int -> Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
dblClick dwCharID fwKeys x y =
  Automation.method0 "DblClick" [Automation.inInt dwCharID, Automation.inInt fwKeys, Automation.inInt x, Automation.inInt y]
dragStart :: Prelude.Int -> Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
dragStart dwCharID fwKeys x y =
  Automation.method0 "DragStart" [Automation.inInt dwCharID, Automation.inInt fwKeys, Automation.inInt x, Automation.inInt y]
dragComplete :: Prelude.Int -> Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
dragComplete dwCharID fwKeys x y =
  Automation.method0 "DragComplete" [Automation.inInt dwCharID, Automation.inInt fwKeys, Automation.inInt x, Automation.inInt y]
requestStart :: Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
requestStart dwRequestID =
  Automation.method0 "RequestStart" [Automation.inInt dwRequestID]
requestComplete :: Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
requestComplete dwRequestID hrStatus =
  Automation.method0 "RequestComplete" [Automation.inInt dwRequestID, Automation.inInt hrStatus]
bookMark :: Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
bookMark dwBookMarkID =
  Automation.method0 "BookMark" [Automation.inInt dwBookMarkID]
idle :: Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
idle dwCharID bStart =
  Automation.method0 "Idle" [Automation.inInt dwCharID, Automation.inInt bStart]
move :: Prelude.Int -> Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
move dwCharID x y dwCause =
  Automation.method0 "Move" [Automation.inInt dwCharID, Automation.inInt x, Automation.inInt y, Automation.inInt dwCause]
size :: Prelude.Int -> Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
size dwCharID lWidth lHeight =
  Automation.method0 "Size" [Automation.inInt dwCharID, Automation.inInt lWidth, Automation.inInt lHeight]
balloonVisibleState :: Prelude.Int -> Prelude.Int -> IAgentNotifySink a -> Prelude.IO ()
balloonVisibleState dwCharID bVisible =
  Automation.method0 "BalloonVisibleState" [Automation.inInt dwCharID, Automation.inInt bVisible]
-- --------------------------------------------------
-- 
-- interface IAgentCommandWindow
-- 
-- --------------------------------------------------
data AgentCommandWindow a = AgentCommandWindow
                             
type IAgentCommandWindow a = Automation.IDispatch (AgentCommandWindow a)
iidIAgentCommandWindow :: Com.IID (IAgentCommandWindow Automation.Interface)
iidIAgentCommandWindow =
  Com.mkIID "{6D0ECB23-9968-11D0-AC6E-00C04FD97575}"
