\documentclass{article}
\include{/hugs/lib/activex/lhs}

\begin{document}
\begin{notice}
Copyright (c) 1998,  Daan Leijen, leijen@@fwi.uva.nl

This module is part of HaskellScript (HScript).
See @notice.txt@ for details and conditions of use.
\end{notice}

> module Automation (
>       module Com,

>       Dispatch, IDispatch, iidIDispatch, 
>       IAutomation, iidIAutomation,

>       createObject, getObject', getActiveObject, getFileObject,

>       Member, DISPID, getMemberID, VARIANT, sizeVARIANT,
>       VarIn, VarRes, ArgIn, ArgInOut, ArgOut, ArgRes,

>       Variant(..),
>       outVariant, inoutVariant,
>       queryIUnknown, queryIDispatch,

>       defaultEmpty, inEmpty, resEmpty, inoutEmpty, outEmpty,
>       defaultInt, inInt, resInt, inoutInt, outInt,
>       defaultWord, inWord, resWord, inoutWord, outWord,
>       defaultBool, inBool, resBool, inoutBool, outBool,
>       defaultByte, inByte, resByte, inoutByte, outByte,
>       defaultFloat, inFloat, resFloat, inoutFloat, outFloat,
>       defaultDouble, inDouble, resDouble, inoutDouble, outDouble,
>       defaultString, inString, resString, inoutString, outString,
>       defaultIUnknown, inIUnknown, resIUnknown, inoutIUnknown, outIUnknown,
>       defaultIDispatch, inIDispatch, resIDispatch, inoutIDispatch, outIDispatch,
>       defaultDate, inDate, resDate, inoutDate, outDate, Date,
>       defaultError, inError, resError, inoutError, outError,
>       defaultOptional, inOptional, resOptional, inoutOptional, 
>
>       propertyGet, propertySet,
>       propertyGetID, propertySetID,

>       function1, function2, function3, function4, function5,
>       functionID1, functionID2, functionID3, functionID4, functionID5,

>       method0, method1, method2, method3, method4,
>       method5,
>       methodID0, methodID1, methodID2, methodID3, methodID4,
>       methodID5, methodID6, methodID7, methodID8,

>       unmarshallVariants0, unmarshallVariants1,
>       unmarshallVariants2, unmarshallVariants3,
>       unmarshallVariants4, unmarshallVariants5,
>       unmarshallVariants6, unmarshallVariants7,
>       unmarshallVariants8,

>       readVariants0, readVariants1,
>       readVariants2, readVariants3,
>       readVariants4, readVariants5,
>       readVariants6, readVariants7,
>       readVariants8,

>       method_0_0, method_1_0, method_2_0, method_3_0, method_4_0,
>       method_0_1, method_1_1, method_2_1, method_3_1, method_4_1,
>       method_0_2, method_1_2, method_2_2, method_3_2, method_4_2,

>       function_0_1, function_1_1, function_2_1, function_3_1, function_4_1,
>       function_0_2, function_1_2, function_2_2, function_3_2, function_4_2,

>       propertyGet_0, propertyGet_1, propertyGet_2, propertyGet_3, propertyGet_4,
>       propertySet_1, propertySet_2, propertySet_3, propertySet_4,

>       invokePropertyGet, invokePropertyGet,
>       invokeMethod, invokeFunction,

>       allocBSTR, freeBSTR, readBSTRAt,

>       enumVariants

> ) where

> import LibIO   ( always )
> import Pointer ( Pointer,
>                  allocMemory,stackFrame,
>                  pointerInc, writeSeqAtDec, 
>                  freeMemory, freeBSTR
>                )
> import Marshall ( stackString, stackStringLen, readStringAt
>                )
> import IO (ioeGetErrorString)
> import Com
> import AutoPrim


The @IDispatch@ data type.

> iidIDispatch :: IID (IDispatch Interface)
> iidIDispatch            = mkIID "{00020400-0000-0000-C000-000000000046}"

An 'unsafe' version of the @IDispatch@ IID is provided
with the @iidIAutomation@ function. When querying for
an automation interface, the result can be used an ANY dispatch
interface. It is possible to get run-time errors like:
'method X called but not supported'.

> type IAutomation a      = IDispatch a
>
> iidIAutomation :: IID (IDispatch a)
> iidIAutomation          = mkIID "{00020400-0000-0000-C000-000000000046}"


The following creation functions are the VB equivalents of Haskell,
the use of the @iidIAutomation@ makes them just as dynamic as in VB.

> createObject  :: ProgID -> IO (IAutomation a)
> createObject progid
>       = coCreateObject progid iidIAutomation
>
> getObject' :: String -> ProgID -> IO (IAutomation a)
> getObject' fname progid
>       = coGetObject fname progid iidIAutomation
>
> getActiveObject :: ProgID -> IO (IAutomation a)
> getActiveObject progid
>       = coGetActiveObject progid iidIAutomation
>
> getFileObject :: String -> IO (IAutomation a)
> getFileObject fname
>       = coGetFileObject fname iidIAutomation
>

The following functions are overloaded versions of the basic functions.
The postfix "_n_m" means n input arguments and m results.

> method_0_0 name               = method0 name []
> method_1_0 name a1            = method0 name [inVariant a1] 
> method_2_0 name a1 a2         = method0 name [inVariant a1, inVariant a2]
> method_3_0 name a1 a2 a3      = method0 name [inVariant a1, inVariant a2, inVariant a3]
> method_4_0 name a1 a2 a3 a4   = method0 name [inVariant a1, inVariant a2, 
>                                               inVariant a3, inVariant a4]

> method_0_1 name               = method1 name [] outVariant
> method_1_1 name a1            = method1 name [inVariant a1] outVariant
> method_2_1 name a1 a2         = method1 name [inVariant a1, inVariant a2] outVariant
> method_3_1 name a1 a2 a3      = method1 name [inVariant a1, inVariant a2, 
>                                               inVariant a3] outVariant
> method_4_1 name a1 a2 a3 a4   = method1 name [inVariant a1, inVariant a2, 
>                                               inVariant a3, inVariant a4] outVariant

> method_0_2 name               = method2 name [] outVariant outVariant
> method_1_2 name a1            = method2 name [inVariant a1] outVariant outVariant
> method_2_2 name a1 a2         = method2 name [inVariant a1, inVariant a2] outVariant outVariant
> method_3_2 name a1 a2 a3      = method2 name [inVariant a1, inVariant a2, 
>                                               inVariant a3] outVariant outVariant
> method_4_2 name a1 a2 a3 a4   = method2 name [inVariant a1, inVariant a2, 
>                                               inVariant a3, inVariant a4] outVariant outVariant

> function_0_1 name               = function1 name [] outVariant
> function_1_1 name a1            = function1 name [inVariant a1] outVariant
> function_2_1 name a1 a2         = function1 name [inVariant a1, inVariant a2] outVariant
> function_3_1 name a1 a2 a3      = function1 name [inVariant a1, inVariant a2, 
>                                               inVariant a3] outVariant
> function_4_1 name a1 a2 a3 a4   = function1 name [inVariant a1, inVariant a2, 
>                                               inVariant a3, inVariant a4] outVariant

> function_0_2 name               = function2 name [] outVariant outVariant
> function_1_2 name a1            = function2 name [inVariant a1] outVariant outVariant
> function_2_2 name a1 a2         = function2 name [inVariant a1, inVariant a2] 
>                                                   outVariant outVariant
> function_3_2 name a1 a2 a3      = function2 name [inVariant a1, inVariant a2, 
>                                                   inVariant a3] outVariant outVariant
> function_4_2 name a1 a2 a3 a4   = function2 name [inVariant a1, inVariant a2, 
>                                                   inVariant a3, inVariant a4] 
>                                                   outVariant outVariant

> propertyGet_0 name              = propertyGet name [] outVariant
> propertyGet_1 name a1           = propertyGet name [inVariant a1] outVariant
> propertyGet_2 name a1 a2        = propertyGet name [inVariant a1, inVariant a2] outVariant
> propertyGet_3 name a1 a2 a3     = propertyGet name [inVariant a1, inVariant a2,
>                                                     inVariant a3] outVariant
> propertyGet_4 name a1 a2 a3 a4  = propertyGet name [inVariant a1, inVariant a2,
>                                                     inVariant a3, inVariant a4] outVariant

> propertySet_1 name a1           = propertySet name [inVariant a1]
> propertySet_2 name a1 a2        = propertySet name [inVariant a1, inVariant a2]
> propertySet_3 name a1 a2 a3     = propertySet name [inVariant a1, inVariant a2, inVariant a3]
> propertySet_4 name a1 a2 a3 a4  = propertySet name [inVariant a1, inVariant a2,
>                                                     inVariant a3, inVariant a4]


Automation member functions or properties are identified
by name or DISPID.

> type Member        = String
> type DISPID        = Int
> sizeDISPID         = 4
>
> getMemberID :: Member -> IDispatch a -> IO DISPID
> getMemberID name obj
>      = do bstr        <- allocBSTR name
>           (hr,dispid) <- dispatchGetMemberID obj bstr lcidNeutral
>                           `always` freeBSTR bstr
>           checkHR hr  `catch` \err -> if (dispErrorUNKNOWNNAME hr)
>                                        then coFail ("method '" ++ name ++
>                                               "' called but not supported by object")
>                                        else errorMember name err
>           return dispid

Type definitions for marshalling functions. Variants are represented
as functions that can read or write a value from or to a variant structure.

> sizeVARIANT           = 16

> type VarIn            = Pointer VARIANT -> IO ()
> type VarRes a         = Pointer VARIANT -> IO a

> type ArgIn a          = a -> VarIn
> type ArgRes a         = VarRes a
> type ArgOut a         = (VarIn,ArgRes a)
> type ArgInOut a       = a -> ArgOut a


For each type we define 5 functions; @defaultT@, @inT@,
@resT@, @inoutT@, @outT@ where the last
two functions are defined in terms of the first three.
The @Variant@ class overloads these functions; if an
argument can be more than one type, it will be overloaded
and Haskell takes care of resolving the marshall function to use.
We enable overlapping instance by providing explicit constructor functions
in the class definition.

Input variants.

> class Variant a where
>   inVariant :: ArgIn a
>   inVarList :: ArgIn [a]
>   inVarIUnknown :: ArgIn (IUnknown a)
>
>   resVariant :: ArgRes a
>   defaultVariant :: a
>
>   resVarList :: ArgRes [a]
>   resVarIUnknown :: ArgRes (IUnknown a)
>   resVarIDispatch :: ArgRes (IDispatch a)

Overlapping instance for strings.

> instance Variant a => Variant [a] where
>   inVariant       = inVarList
>   resVariant      = resVarList
>   defaultVariant  = []

>
> instance Variant Char where
>   inVarList   = inString
>   resVarList  = resString
>

Overlapping instance for @IDispatch a@ and @IUnknown Interface@
variants.

> instance Variant a => Variant (Unknown a) where
>   inVariant   = inVarIUnknown
>   resVariant  = resVarIUnknown
>   defaultVariant = defaultIUnknown
>
> instance Variant (Dispatch a) where
>   inVarIUnknown  = inIDispatch
>   resVarIUnknown = resIDispatch
>
> instance Variant () where
>   inVarIUnknown   = inIUnknown
>   resVarIUnknown  = resIUnknown
>   resVarIDispatch = resIDispatch
>
>   inVariant   = inEmpty
>   resVariant  = resEmpty
>   defaultVariant = defaultEmpty

Normal instances.

> instance Variant Bool where
>   inVariant   = inBool
>   resVariant  = resBool
>   defaultVariant = defaultBool
>
> instance Variant Int where
>   inVariant   = inInt
>   resVariant  = resInt
>   defaultVariant = defaultInt
>
> instance Variant Float where
>   inVariant   = inFloat
>   resVariant  = resFloat
>   defaultVariant = defaultFloat
>
> instance Variant Double where
>   inVariant   = inDouble
>   resVariant  = resDouble
>   defaultVariant = defaultDouble

> instance Variant a => Variant (Maybe a) where
>   inVariant   = inOptional
>   resVariant  = resOptional
>   defaultVariant = defaultOptional


> inoutVariant :: (Variant a) => ArgInOut a
> inoutVariant x        = (inVariant x,resVariant)
>
> outVariant :: (Variant a) => ArgOut a
> outVariant            = (inoutVariant defaultVariant)

> queryIUnknown :: IID (IUnknown a) -> IUnknown Interface -> IO (IUnknown a)
> queryIUnknown          = queryInterface

> queryIDispatch :: IID (IUnknown a) -> IDispatch Interface -> IO (IUnknown a)
> queryIDispatch         = queryInterface


The basic marshalling functions for automation types.

> defaultEmpty :: ()
> defaultEmpty            = ()
>
> inEmpty :: ArgIn ()
> inEmpty i               = writeVarEmpty 
>
> resEmpty :: ArgRes ()
> resEmpty p              = return ()
>
> inoutEmpty e            = (inEmpty e,resEmpty)
> outEmpty                = inoutEmpty defaultEmpty




Integers.

> defaultInt :: Int
> defaultInt            = 0
>
> inInt :: ArgIn Int
> inInt i               = writeVarInt i
>
> resInt :: ArgRes Int
> resInt                = readVarInt
>
> inoutInt i            = (inInt i,resInt)
> outInt                = inoutInt defaultInt

Words

> defaultWord :: Int
> defaultWord            = 0
>
> inWord :: ArgIn Int
> inWord i               = writeVarInt i
>
> resWord :: ArgRes Int
> resWord                = readVarInt
>
> inoutWord i            = (inInt i,resInt)
> outWord                = inoutInt defaultInt

Bytes

> defaultByte :: Byte
> defaultByte            = '\0'
>
> inByte :: ArgIn Byte
> inByte i               = writeVarByte i
>
> resByte :: ArgRes Byte
> resByte                = readVarByte
>
> inoutByte i            = (inByte i,resByte)
> outByte                = inoutByte defaultByte

Booleans.

> defaultBool :: Bool
> defaultBool            = False
>
> inBool :: ArgIn Bool
> inBool b               = writeVarBool b
>
> resBool :: ArgRes Bool
> resBool v              = readVarBool v
>
> inoutBool b            = (inBool b,resBool)
> outBool                = inoutBool defaultBool


Floats.

> defaultFloat :: Float
> defaultFloat           = 0.0
>
> inFloat :: ArgIn Float
> inFloat f              = writeVarFloat f
>
> resFloat :: ArgRes Float
> resFloat               = readVarFloat
>
> inoutFloat b           = (inFloat b,resFloat)
> outFloat               = inoutFloat defaultFloat

Doubles.

> defaultDouble :: Double
> defaultDouble           = 0.0
>
> inDouble :: ArgIn Double
> inDouble f              = writeVarDouble f
>
> resDouble :: ArgRes Double
> resDouble               = readVarDouble
>
> inoutDouble b           = (inDouble b,resDouble)
> outDouble               = inoutDouble defaultDouble

Dates.

>
> type Date = Double
> 
> defaultDate :: Date
> defaultDate           = 0.0
>
> inDate :: ArgIn Date
> inDate f              = writeVarDouble f
>
> resDate :: ArgRes Date
> resDate               = readVarDouble
>
> inoutDate b           = (inDate b,resDate)
> outDate               = inoutDate defaultDate



Strings.

> defaultString :: String
> defaultString            = ""
>
> inString :: ArgIn String
> inString s p             = do bstr <- nofreeAllocBSTR s
>                               writeVarString bstr p
>
> resString :: ArgRes String
> resString p              = readTempVar readVarString p $ \pbstr ->
>                            do readBSTRAt pbstr
>
> inoutString i            = (inString i,resString)
> outString                = inoutString defaultString

Unknown objects.

> defaultIUnknown :: IUnknown a
> defaultIUnknown          = interfaceNULL
>
> inIUnknown :: ArgIn (IUnknown a)
> inIUnknown u p          = do u # addRef    -- u could be NULL !
>                              writeVarUnknown u p
>
>
> resIUnknown :: ArgRes (IUnknown a)
> resIUnknown p           = readTempVar readVarUnknown p $ \punk ->
>                           do punk # addRef
>                              return punk
>
> inoutIUnknown d          = (inIUnknown d,resIUnknown)
> outIUnknown              = inoutIUnknown defaultIUnknown


Dispatch objects.

> defaultIDispatch :: IDispatch a
> defaultIDispatch          = interfaceNULL
>
> inIDispatch :: ArgIn (IDispatch a)
> inIDispatch d p          = do d # addRef    -- d could be NULL !
>                               writeVarDispatch d p
>
>
> resIDispatch :: ArgRes (IDispatch a)
> resIDispatch p           = readTempVar readVarDispatch p $ \pdisp ->
>                            do pdisp # addRef
>                               return pdisp
>
> inoutIDispatch d          = (inIDispatch d,resIDispatch)
> outIDispatch              = inoutIDispatch defaultIDispatch


0x80020004

Error objects.

> defaultError :: Int
> defaultError         = 0
>
> inError :: ArgIn Int
> inError d p          = writeVarError d p
>
> resError :: ArgRes Int
> resError p           = readVarError p
>
> inoutError d          = (inError d,resError)
> outError              = inoutError defaultError


> defaultOptional :: Variant a => Maybe a
> defaultOptional       = Nothing
>
> inOptional :: Variant a => ArgIn (Maybe a)
> inOptional Nothing    = inError (-2147352572)
> inOptional (Just x)   = inVariant x
>
> resOptional :: Variant a => ArgRes (Maybe a)
> resOptional p         = (readVarError p >> return Nothing) `catch` \err ->           
>                         (map Just (resVariant p))
>
> inoutOptional o       = (inOptional o,resOptional)
> --outOptional :: Variant a => (VarIn,ArgRes (Maybe a))
> --outOptional           = inoutOptional defaultOptional
                    

Setting and Getting properties: @getVisible = propertyGet "Visible" [] outBool@.

> propertyGet :: Member -> [VarIn] -> ArgOut a -> IDispatch d -> IO a
> propertyGet member argsin argout obj
>       = do dispid <- obj # getMemberID member
>            propertyGetID dispid argsin argout obj
>
> propertySet :: Member -> [VarIn] -> IDispatch d -> IO ()
> propertySet member argsin obj
>       = do dispid <- obj # getMemberID member
>            propertySetID dispid argsin obj

> propertyGetID :: DISPID -> [VarIn] -> ArgOut a -> IDispatch d -> IO a
> propertyGetID dispid argsin (varin,argres) obj
>       = do p <- obj # invokePropertyGet dispid argsin [varin]
>            unmarshallVariants1 argres p
>
> propertySetID :: DISPID -> [VarIn] -> IDispatch d -> IO ()
> propertySetID dispid argsin obj
>       = do p <- obj # invokePropertySet dispid argsin []
>            unmarshallVariants0 p


Methods and functions are defined using @method@/@funtion@. The digit
appended to the name gives the number of results.
For example: @confirm msg  = function1 "Confirm" [inString msg] outBool@.

> method0 :: Member -> [VarIn] -> IDispatch d -> IO ()
> method0 member args obj
>      = do dispid <- obj # getMemberID member
>           methodID0 dispid args obj
>               `catch` errorMember member
>
> method1 :: Member -> [VarIn] -> ArgOut a -> IDispatch d -> IO a
> method1 member args argout obj
>      = do dispid <- obj # getMemberID member
>           methodID1 dispid args argout obj
>               `catch` errorMember member
>
> method2 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> IDispatch d -> IO (a,b)
> method2 member args argout1 argout2 obj
>      = do dispid <- obj # getMemberID member
>           methodID2 dispid args argout1 argout2 obj
>               `catch` errorMember member
>
> method3 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> ArgOut c
>               -> IDispatch d -> IO (a,b,c)
> method3 member args argout1 argout2 argout3 obj
>      = do dispid <- obj # getMemberID member
>           methodID3 dispid args argout1 argout2 argout3 obj
>               `catch` errorMember member
>
> method4 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> ArgOut c -> ArgOut d
>               -> IDispatch e -> IO (a,b,c,d)
> method4 member args argout1 argout2 argout3 argout4 obj
>      = do dispid <- obj # getMemberID member
>           methodID4 dispid args argout1 argout2 argout3 argout4 obj
>               `catch` errorMember member
>
> method5 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> ArgOut c -> ArgOut d
>               -> ArgOut e
>               -> IDispatch f -> IO (a,b,c,d,e)
> method5 member args argout1 argout2 argout3 argout4 argout5 obj
>      = do dispid <- obj # getMemberID member
>           methodID5 dispid args argout1 argout2 argout3 argout4 argout5 obj
>               `catch` errorMember member



Methods invoked on DISPID.

> methodID0 :: DISPID -> [VarIn] -> IDispatch d -> IO ()
> methodID0 dispid args obj
>      = do p <- obj # invokeMethod dispid args []
>           unmarshallVariants0 p
>
> methodID1 :: DISPID -> [VarIn] -> ArgOut b -> IDispatch d -> IO b
> methodID1 dispid args (varin,argres) obj
>       = do p <- obj # invokeMethod dispid args [varin]
>            unmarshallVariants1 argres p
>
> methodID2 :: DISPID -> [VarIn]
>             -> ArgOut a -> ArgOut b -> IDispatch d -> IO (a,b)
> methodID2 dispid args (varin1,argres1) (varin2,argres2) obj
>      = do p <- obj # invokeMethod dispid args [varin1,varin2]
>           unmarshallVariants2 argres1 argres2 p
>
> methodID3 :: DISPID -> [VarIn]
>             -> ArgOut a -> ArgOut b -> ArgOut c -> IDispatch d -> IO (a,b,c)
> methodID3 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) obj
>      = do p <- obj # invokeMethod dispid args [varin1,varin2,varin3]
>           unmarshallVariants3 argres1 argres2 argres3 p
>
> methodID4 :: DISPID -> [VarIn]
>             -> ArgOut a -> ArgOut b -> ArgOut c -> ArgOut d
>             -> IDispatch e -> IO (a,b,c,d)
> methodID4 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) (varin4,argres4) obj
>      = do p <- obj # invokeMethod dispid args [varin1,varin2,varin3,varin4]
>           unmarshallVariants4 argres1 argres2 argres3 argres4 p
> 
> methodID5 :: DISPID -> [VarIn]
>             -> ArgOut a1 -> ArgOut a2 -> ArgOut a3 -> ArgOut a4 -> ArgOut a5
>             -> IDispatch n -> IO (a1,a2,a3,a4,a5)
> methodID5 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) (varin4,argres4) (varin5,argres5) obj
>      = do p <- obj # invokeMethod dispid args [varin1,varin2,varin3,varin4,varin5]
>           unmarshallVariants5 argres1 argres2 argres3 argres4 argres5 p
> 
> 
> methodID6 :: DISPID -> [VarIn]
>             -> ArgOut a1 -> ArgOut a2 -> ArgOut a3 -> ArgOut a4 -> ArgOut a5 -> ArgOut a6
>             -> IDispatch n -> IO (a1,a2,a3,a4,a5,a6)
> methodID6 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) (varin4,argres4) (varin5,argres5) (varin6,argres6) obj
>      = do p <- obj # invokeMethod dispid args [varin1,varin2,varin3,varin4,varin5,varin6]
>           unmarshallVariants6 argres1 argres2 argres3 argres4 argres5 argres6 p
> 
> methodID7 :: DISPID -> [VarIn]
>             -> ArgOut a1 -> ArgOut a2 -> ArgOut a3 -> ArgOut a4 
>             -> ArgOut a5 -> ArgOut a6 -> ArgOut a7
>             -> IDispatch n -> IO (a1,a2,a3,a4,a5,a6,a7)
> methodID7 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) (varin4,argres4) (varin5,argres5) (varin6,argres6) (varin7,argres7) obj
>      = do p <- obj # invokeMethod dispid args [varin1,varin2,varin3,varin4,varin5,varin6,varin7]
>           unmarshallVariants7 argres1 argres2 argres3 argres4 argres5 argres6 argres7 p
> 
> methodID8 :: DISPID -> [VarIn]
>             -> ArgOut a1 -> ArgOut a2 -> ArgOut a3 -> ArgOut a4 
>             -> ArgOut a5 -> ArgOut a6 -> ArgOut a7 -> ArgOut a8
>             -> IDispatch n -> IO (a1,a2,a3,a4,a5,a6,a7,a8)
> methodID8 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) (varin4,argres4) (varin5,argres5) (varin6,argres6) (varin7,argres7) (varin8,argres8) obj
>      = do p <- obj # invokeMethod dispid args [varin1,varin2,varin3,varin4,varin5,varin6,varin7,varin8]
>           unmarshallVariants8 argres1 argres2 argres3 argres4 argres5 argres6 argres7 argres8 p


Functions. Of course @function0@ is missing. The difference with
methods is that functions expect the last @out@ argument to be
a result (@retval@) instead of a real @out@ argument.

> function1 :: Member -> [VarIn] -> ArgOut a -> IDispatch d -> IO a
> function1 member args argout obj
>      = do dispid <- obj # getMemberID member
>           functionID1 dispid args argout obj
>               `catch` errorMember member
>
> function2 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> IDispatch d -> IO (a,b)
> function2 member args argout1 argout2 obj
>      = do dispid <- obj # getMemberID member
>           functionID2 dispid args argout1 argout2 obj
>               `catch` errorMember member
>
> function3 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> ArgOut c -> IDispatch d -> IO (a,b,c)
> function3 member args argout1 argout2 argout3 obj
>      = do dispid <- obj # getMemberID member
>           functionID3 dispid args argout1 argout2 argout3 obj
>               `catch` errorMember member
>
> function4 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> ArgOut c -> ArgOut d -> IDispatch e -> IO (a,b,c,d)
> function4 member args argout1 argout2 argout3 argout4 obj
>      = do dispid <- obj # getMemberID member
>           functionID4 dispid args argout1 argout2 argout3 argout4 obj
>               `catch` errorMember member
>
> function5 :: Member -> [VarIn] -> ArgOut a -> ArgOut b -> ArgOut c -> ArgOut d -> ArgOut e -> IDispatch f -> IO (a,b,c,d,e)
> function5 member args argout1 argout2 argout3 argout4 argout5 obj
>      = do dispid <- obj # getMemberID member
>           functionID5 dispid args argout1 argout2 argout3 argout4 argout5 obj
>               `catch` errorMember member

> functionID1 :: DISPID -> [VarIn] -> ArgOut b -> IDispatch d -> IO b
> functionID1 dispid args (varin,argres) obj
>       = do p <- obj # invokeFunction dispid args [varin]
>            unmarshallVariants1 argres p
>
> functionID2 :: DISPID -> [VarIn]
>             -> ArgOut a -> ArgOut b -> IDispatch d -> IO (a,b)
> functionID2 dispid args (varin1,argres1) (varin2,argres2) obj
>      = do p <- obj # invokeFunction dispid args [varin1,varin2]
>           unmarshallVariants2 argres1 argres2 p
>
> functionID3 :: DISPID -> [VarIn]
>             -> ArgOut a -> ArgOut b -> ArgOut c -> IDispatch d -> IO (a,b,c)
> functionID3 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) obj
>      = do p <- obj # invokeFunction dispid args [varin1,varin2,varin3]
>           unmarshallVariants3 argres1 argres2 argres3 p
>
> functionID4 :: DISPID -> [VarIn]
>             -> ArgOut a -> ArgOut b -> ArgOut c -> ArgOut d -> IDispatch e -> IO (a,b,c,d)
> functionID4 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) (varin4,argres4) obj
>      = do p <- obj # invokeFunction dispid args [varin1,varin2,varin3,varin4]
>           unmarshallVariants4 argres1 argres2 argres3 argres4 p
>
> functionID5 :: DISPID -> [VarIn]
>             -> ArgOut a -> ArgOut b -> ArgOut c -> ArgOut d -> ArgOut e 
>             -> IDispatch f -> IO (a,b,c,d,e)
> functionID5 dispid args (varin1,argres1) (varin2,argres2) (varin3,argres3) (varin4,argres4) (varin5,argres5) obj
>      = do p <- obj # invokeFunction dispid args [varin1,varin2,varin3,varin4,varin5]
>           unmarshallVariants5 argres1 argres2 argres3 argres4 argres5 p


Error reporting:

> errorMember :: String -> IOError -> IO a
> errorMember member err
>       = coFail ("method '" ++ member ++ "': " ++ (coGetErrorString err))

Unmarshall the @out@ arguments.

> unmarshallVariants0 p 
>       = readVariants0 p `always` freeMemVariants 0 p
>
> unmarshallVariants1 a p
>       = readVariants1 a p `always` freeMemVariants 1 p
>
> unmarshallVariants2   a b p
>       = readVariants2 a b p `always` freeMemVariants 2 p
>
> unmarshallVariants3   a b c p
>       = readVariants3 a b c p `always` freeMemVariants 3 p
>
> unmarshallVariants4   a b c d p
>       = readVariants4 a b c d p `always` freeMemVariants 4 p
>
> unmarshallVariants5   a b c d e p
>       = readVariants5 a b c d e p `always` freeMemVariants 5 p
>
> unmarshallVariants6   a b c d e f p
>       = readVariants6 a b c d e f p `always` freeMemVariants 6 p
>
> unmarshallVariants7   a b c d e f g p
>       = readVariants7 a b c d e f g p `always` freeMemVariants 7 p
>
> unmarshallVariants8   a b c d e f g h p
>       = readVariants8 a b c d e f g h p `always` freeMemVariants 8 p
>
>
> readVariants0 :: Pointer VARIANT -> IO ()
> readVariants0 p
>       = return ()
>
> readVariants1 :: ArgRes a -> Pointer VARIANT  -> IO a
> readVariants1 f p
>       = f p
>
> readVariants2 :: ArgRes a -> ArgRes b -> Pointer VARIANT  -> IO (a,b)
> readVariants2 f g p
>       = do y <- g p
>            x <- f (pointerInc sizeVARIANT p)
>            return (x,y)
>
> readVariants3 :: ArgRes a -> ArgRes b -> ArgRes c
>                       -> Pointer VARIANT  -> IO (a,b,c)
> readVariants3 f g h p
>       = do z <- h p
>            y <- g (pointerInc sizeVARIANT p)
>            x <- f (pointerInc (2*sizeVARIANT) p)
>            return (x,y,z)
> 
> readVariants4 :: ArgRes a -> ArgRes b -> ArgRes c -> ArgRes d
>                       -> Pointer VARIANT  -> IO (a,b,c,d)
> readVariants4 f g h i p
>       = do z <- i p
>            y <- h (pointerInc sizeVARIANT p)
>            x <- g (pointerInc (2*sizeVARIANT) p)
>            w <- f (pointerInc (3*sizeVARIANT) p)
>            return (w,x,y,z)
> 
> readVariants5 :: ArgRes a -> ArgRes b -> ArgRes c -> ArgRes d -> ArgRes e
>                       -> Pointer VARIANT  -> IO (a,b,c,d,e)
> readVariants5 f g h i j p
>       = do z <- j p
>            y <- i (pointerInc sizeVARIANT p)
>            x <- h (pointerInc (2*sizeVARIANT) p)
>            w <- g (pointerInc (3*sizeVARIANT) p)
>            v <- f (pointerInc (4*sizeVARIANT) p)
>            return (v,w,x,y,z)
> 
> readVariants6 :: ArgRes a1 -> ArgRes a2 -> ArgRes a3 -> ArgRes a4 -> ArgRes a5 -> ArgRes a6
>                       -> Pointer VARIANT  -> IO (a1,a2,a3,a4,a5,a6)
> readVariants6 f1 f2 f3 f4 f5 f6 p
>       = do v6 <- f6 p
>            v5 <- f5 (pointerInc sizeVARIANT p)
>            v4 <- f4 (pointerInc (2*sizeVARIANT) p)
>            v3 <- f3 (pointerInc (3*sizeVARIANT) p)
>            v2 <- f2 (pointerInc (4*sizeVARIANT) p)
>            v1 <- f1 (pointerInc (5*sizeVARIANT) p)
>            return (v1,v2,v3,v4,v5,v6)
> 
> readVariants7 :: ArgRes a1 -> ArgRes a2 -> ArgRes a3 -> ArgRes a4 -> ArgRes a5 
>                       -> ArgRes a6 -> ArgRes a7
>                       -> Pointer VARIANT  -> IO (a1,a2,a3,a4,a5,a6,a7)
> readVariants7 f1 f2 f3 f4 f5 f6 f7 p
>       = do v7 <- f7 p
>            v6 <- f6 (pointerInc sizeVARIANT p)
>            v5 <- f5 (pointerInc (2*sizeVARIANT) p)
>            v4 <- f4 (pointerInc (3*sizeVARIANT) p)
>            v3 <- f3 (pointerInc (4*sizeVARIANT) p)
>            v2 <- f2 (pointerInc (5*sizeVARIANT) p)
>            v1 <- f1 (pointerInc (6*sizeVARIANT) p)
>            return (v1,v2,v3,v4,v5,v6,v7)
> 
> readVariants8:: ArgRes a1 -> ArgRes a2 -> ArgRes a3 -> ArgRes a4 -> ArgRes a5 
>                       -> ArgRes a6 -> ArgRes a7 -> ArgRes a8
>                       -> Pointer VARIANT  -> IO (a1,a2,a3,a4,a5,a6,a7,a8)
> readVariants8 f1 f2 f3 f4 f5 f6 f7 f8 p
>       = do v8 <- f8 p
>            v7 <- f7 (pointerInc sizeVARIANT p)
>            v6 <- f6 (pointerInc (2*sizeVARIANT) p)
>            v5 <- f5 (pointerInc (3*sizeVARIANT) p)
>            v4 <- f4 (pointerInc (4*sizeVARIANT) p)
>            v3 <- f3 (pointerInc (5*sizeVARIANT) p)
>            v2 <- f2 (pointerInc (6*sizeVARIANT) p)
>            v1 <- f1 (pointerInc (7*sizeVARIANT) p)
>            return (v1,v2,v3,v4,v5,v6,v7,v8)
> 
> {- UNUSED:
> unmarshallVariantList :: [ArgRes a] -> Pointer VARIANT  -> IO [a]
> unmarshallVariantList fls p = 
>    (go p fls []) `always` freeMemVariants len p
>   where
>     len = length fls
> 
>     go p [] acc     = return acc
>     go p (f:fs) acc = do
>         v <- f p
>         go (pointerInc sizeVARIANT p) fs (v:acc)
> -}

@invokeMethod/Function@ and @propertyGet/Set@ all use the primitive
@primInvokeMethod@.

> invokePropertyGet     = primInvokeMethod dispPROPERTYGET True
> invokePropertySet     = primInvokeMethod dispPROPERTYSET False
> invokeMethod          = primInvokeMethod dispMETHOD False
> invokeFunction        = primInvokeMethod dispMETHOD True

Some constants used with the invoke functions.

> type DispAction         = Int
>
> dispMETHOD              = 1
> dispPROPERTYGET         = 2
> dispPROPERTYSET         = 4
> dispPROPERTYSETREF      = 8
>
> lcidNeutral             = 0

The primitive invokation mechanism. Exceptions are directed to the normal
@coFail@ function.

> primInvokeMethod :: DispAction -> Bool -> DISPID
>                       -> [VarIn] -> [VarIn]
>                       -> IDispatch d -> IO (Pointer VARIANT)
> primInvokeMethod action isfunction dispid argin argout obj
>       = let cargsout = length argout
>             cargs    = length argin + cargsout
>         in
>            stackFrame (sizeVARIANT * cargs) $ \pargs ->
>         do
>            pargout <- allocMemory (sizeVARIANT * cargsout)
>            let pargin  = pointerInc (sizeVARIANT * cargsout) pargs
>
>            writeSeqAtDec sizeVARIANT argin  pargin
>            writeSeqAtDec sizeVARIANT argout pargout
>
>            (hr,pinfo) <- dispatchInvoke obj
>                               dispid lcidNeutral isfunction
>                               action cargs cargsout
>                               pargs pargout
>
>            if (succeededHR hr)
>             then return pargout
>             else if (dispErrorEXCEPTION hr) 
>                   then do 
>                           pstr <- getExcepInfoMessage pinfo
>                           str  <- readStringAt pstr
>                           --putMessage str
>                           freeMemory pstr
>                           freeExcepInfo pinfo
>                           freeMemory pinfo
>                           freeMemVariants cargsout pargout
>                           coFail str
>                   else do 
>                           putMessage "invoke failed"
>                           freeMemVariants cargsout pargout
>                           coFailHR hr

Some helper functions for @Variants@.

> readTempVar io p f
>       = do (x,v) <- io p
>            f x  `always` freeMemVariants 1 v

> freeMemVariants count p
>       = do freeVariants count p
>            freeMemory p

Marshall BSTR values. @allocBSTR@ is called @primAllocBSTR@
since it doesn't take care of freeing the string. (If we
just had true foreign objects: @mkPointer xbstr freeBSTR@.)

> allocBSTR :: String -> IO (Pointer BSTR)
> allocBSTR s             = stackString s $ \pstr ->
>                           do bstrFromString pstr
>

> nofreeAllocBSTR :: String -> IO (Pointer BSTR)
> nofreeAllocBSTR s       = stackString s $ \pstr ->
>                           do nofreeBstrFromString pstr
>


> readBSTRAt :: Pointer BSTR -> IO String
> readBSTRAt bstr         = let len  = lenStringFromBSTR bstr
>                           in if (len == 0)
>                               then return ""
>                               else stackStringLen len "" $ \pstr ->
>                                    do stringFromBSTR bstr len pstr
>                                       readStringAt pstr


> data EnumVARIANT a      = EnumVARIANT
> type IEnumVARIANT a     = IUnknown (EnumVARIANT a)
> iidIEnumVARIANT :: IID (IEnumVARIANT Interface)
> iidIEnumVARIANT = mkIID "{00020404-0000-0000-C000-000000000046}"
>
> newEnum :: IDispatch a -> IO (Int, IEnumVARIANT b)
> newEnum ip = do
>   iunk  <- ip   # function1 "_NewEnum" [] outIUnknown
>   ienum <- iunk # queryInterface iidIEnumVARIANT
>   len   <- ip   # propertyGet "length" [] outInt
>   return (len, ienum)
>   

> enumVariants :: Variant a => IDispatch b -> IO [a]
> enumVariants ip = do
>      (len, ienum) <- newEnum ip
>      enumVariants' len ienum
> 
> enumVariants' :: Variant a => Int -> IEnumVARIANT b -> IO [a]
> enumVariants' len ienum = do
>            real_stuff <- mapM (getOne) [1..len]
>            return real_stuff
>   where
>    -- some implementations of IEnumVARIANT doesn't seem to
>    -- support using Next() to get blocks of the enumeration
>    -- at the time, so we grab them one at a time here.
>    getOne v = do
>        pargout <- allocMemory sizeVARIANT
>        ret     <- ienum # enumNext 1 pargout
>        unmarshallVariants1 resVariant pargout

\end{document}
