-----------------------------------------------------------
-- Copyright (c) 1998,  Daan Leijen, leijen@@fwi.uva.nl
-----------------------------------------------------------

module ComPrim where

import StdDIS
import PointerPrim
import MarshallPrim


type Interface          = ()

newtype Unknown a       = Unknown (Pointer Interface)
type IUnknown a         = Unknown a

data PersistFile a      = PersistFile
type IPersistFile a     = IUnknown (PersistFile a)

data Moniker a          = Moniker
type IMoniker a         = IUnknown (Moniker a)

data BindCtx a          = BindCtx
type IBindCtx a         = IUnknown (BindCtx a)

unsafeMkInterface :: Pointer Interface -> IUnknown a
unsafeMkInterface p     = Unknown p

unsafeGetInterfacePointer :: IUnknown a -> Pointer Interface
unsafeGetInterfacePointer (Unknown p)   = p


marshall_iunknown_ :: IUnknown a -> IO (Pointer Interface)
marshall_iunknown_ (Unknown p)          = return p

-- unsafe function !
unmarshall_iunknown_ :: Pointer Interface -> IO (IUnknown a)
unmarshall_iunknown_ p                  = return (Unknown p)


type IID a              = GUID
type CLSID              = GUID
data GUID               = GUID (Pointer StructGUID)
data StructGUID         = StructGUID
type StructCLSID        = StructGUID
type StructIID          = StructGUID
sizeGUID                = 16


marshall_guid_ (GUID p)                 = return p
unmarshall_guid_ p                      = return (GUID p)


-----------------------------------------------------------
-- Misc.
-----------------------------------------------------------

primitive prim_tickCount :: IO (Int)
tickCount :: IO Int
tickCount =
  prim_tickCount >>= \ (res1) ->
  (return (res1))

primitive prim_uniqueInt :: IO (Int)
uniqueInt :: IO Int
uniqueInt =
  prim_uniqueInt >>= \ (res1) ->
  (return (res1))

primitive prim_messageBox :: ForeignObj -> IO ()
messageBox :: Pointer String -> IO ()
messageBox gc_arg1 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  prim_messageBox arg1

primitive prim_primPlaySound :: ForeignObj -> IO ()
primPlaySound :: Pointer String -> IO ()
primPlaySound gc_arg1 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  prim_primPlaySound arg1

-----------------------------------------------------------
-- Com initialize.
-----------------------------------------------------------

primitive prim_coInitialize :: IO (Int,Addr)
coInitialize :: IO ()
coInitialize =
  prim_coInitialize >>= \ (gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (return (()))

primitive prim_coUnInitialize :: IO (Int,Addr)
coUnInitialize :: IO ()
coUnInitialize =
  prim_coUnInitialize >>= \ (gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (return (()))

-----------------------------------------------------------
-- GUID's
-----------------------------------------------------------

primitive prim_createGUID :: IO (Addr,Addr,Int,Addr)
createGUID :: IO GUID
createGUID =
  prim_createGUID >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_guid_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primGUIDFromString :: ForeignObj -> IO (Addr,Addr,Int,Addr)
primGUIDFromString :: Pointer WideString -> IO GUID
primGUIDFromString gc_arg1 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  prim_primGUIDFromString arg1 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_guid_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primStringFromGUID :: ForeignObj -> IO (Addr,Addr,Int,Addr)
primStringFromGUID :: GUID -> IO (Pointer String)
primStringFromGUID gc_arg1 =
  (marshall_guid_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_primStringFromGUID arg1 >>= \ (gc_res4,gc_res2,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
       (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primProgIDFromCLSID :: ForeignObj -> IO (Addr,Addr,Int,Addr)
primProgIDFromCLSID :: CLSID -> IO (Pointer WideString)
primProgIDFromCLSID gc_arg1 =
  (marshall_guid_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_primProgIDFromCLSID arg1 >>= \ (gc_res4,gc_res2,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
       (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primCLSIDFromProgID :: ForeignObj -> IO (Addr,Addr,Int,Addr)
primCLSIDFromProgID :: Pointer String -> IO CLSID
primCLSIDFromProgID gc_arg1 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  prim_primCLSIDFromProgID arg1 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_guid_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

-----------------------------------------------------------
-- Object creation
-----------------------------------------------------------

primitive prim_primCreateInstance :: ForeignObj -> ForeignObj -> Int -> ForeignObj -> IO (Addr,Addr,Int,Addr)
primCreateInstance :: CLSID -> IUnknown a -> Int -> IID (IUnknown b) -> IO (IUnknown b)
primCreateInstance gc_arg1 gc_arg3 arg3 gc_arg5 =
  (marshall_guid_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  (marshall_iunknown_ gc_arg3) >>= \ (gc_arg4) ->
  (marshall_pointer_ gc_arg4) >>= \ (arg2) ->
  (marshall_guid_ gc_arg5) >>= \ (gc_arg6) ->
  (marshall_pointer_ gc_arg6) >>= \ (arg4) ->
  prim_primCreateInstance arg1 arg2 arg3 arg4 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_persistfileLoad :: ForeignObj -> ForeignObj -> IO (Int,Addr)
persistfileLoad :: IPersistFile a -> Pointer WideString -> IO ()
persistfileLoad gc_arg1 gc_arg3 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  (marshall_pointer_ gc_arg3) >>= \ (arg2) ->
  prim_persistfileLoad arg1 arg2 >>= \ (gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (return (()))

primitive prim_bindObject :: ForeignObj -> ForeignObj -> IO (Addr,Addr,Int,Addr)
bindObject :: Pointer WideString -> IID (IUnknown a) -> IO (IUnknown a)
bindObject gc_arg1 gc_arg2 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  (marshall_guid_ gc_arg2) >>= \ (gc_arg3) ->
  (marshall_pointer_ gc_arg3) >>= \ (arg2) ->
  prim_bindObject arg1 arg2 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primGetActiveObject :: ForeignObj -> IO (Addr,Addr,Int,Addr)
primGetActiveObject :: CLSID -> IO (IUnknown Interface)
primGetActiveObject gc_arg1 =
  (marshall_guid_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_primGetActiveObject arg1 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))


-----------------------------------------------------------
-- HRESULT
-----------------------------------------------------------

primitive prim_failedHR :: Int -> IO (Int)
failedHR :: Int -> Bool
failedHR arg1 =
  unsafePerformIO(
    prim_failedHR arg1 >>= \ (res1) ->
    (unmarshall_bool_ res1) >>= \ gc_res1 ->
    (return (gc_res1)))
primitive prim_winErrorToHR :: Int -> IO (Int)
winErrorToHR :: Int -> Int
winErrorToHR arg1 =
  unsafePerformIO(
    prim_winErrorToHR arg1 >>= \ (res1) ->
    (return (res1)))
primitive prim_primStringFromHR :: Int -> IO (Addr,Addr)
primStringFromHR :: Int -> IO (Pointer String)
primStringFromHR arg1 =
  prim_primStringFromHR arg1 >>= \ (gc_res4,gc_res2) ->
  (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
  (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
  (return (gc_res1))

-----------------------------------------------------------
-- IUnknown
-----------------------------------------------------------

primitive prim_queryInterface :: ForeignObj -> ForeignObj -> IO (Addr,Addr,Int,Addr)
queryInterface :: IID (IUnknown b) -> IUnknown a -> IO (IUnknown b)
queryInterface gc_arg1 gc_arg3 =
  (marshall_guid_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  (marshall_iunknown_ gc_arg3) >>= \ (gc_arg4) ->
  (marshall_pointer_ gc_arg4) >>= \ (arg2) ->
  prim_queryInterface arg1 arg2 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_addRef :: ForeignObj -> IO (Int)
addRef :: IUnknown a -> IO Int
addRef gc_arg1 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_addRef arg1 >>= \ (res1) ->
  (return (res1))

primitive prim_release :: ForeignObj -> IO (Int)
release :: IUnknown a -> IO Int
release gc_arg1 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_release arg1 >>= \ (res1) ->
  (return (res1))

primitive prim_comEqual :: ForeignObj -> ForeignObj -> IO (Int)
comEqual :: IUnknown a -> IUnknown b -> Bool
comEqual gc_arg1 gc_arg3 =
  unsafePerformIO(
    (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
    (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
    (marshall_iunknown_ gc_arg3) >>= \ (gc_arg4) ->
    (marshall_pointer_ gc_arg4) >>= \ (arg2) ->
    prim_comEqual arg1 arg2 >>= \ (res1) ->
    (unmarshall_bool_ res1) >>= \ gc_res1 ->
    (return (gc_res1)))

-----------------------------------------------------------
-- IEnum* methods
--
-- (not type safe, code that uses this better make sure
--  that the interface pointer passed is an IEnum* interface
--  (of the right shape!))
-----------------------------------------------------------

primitive prim_primEnumClone :: ForeignObj -> IO (Addr,Addr,Int,Addr)
primEnumClone :: IUnknown a -> IO (IUnknown b)
primEnumClone gc_arg1 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_primEnumClone arg1 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primEnumNext :: ForeignObj -> Int -> ForeignObj -> IO (Int,Int,Addr)
primEnumNext :: IUnknown a -> Int -> Pointer b -> IO Int
primEnumNext gc_arg1 arg2 gc_arg3 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  (marshall_pointer_ gc_arg3) >>= \ (arg3) ->
  prim_primEnumNext arg1 arg2 arg3 >>= \ (count,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (return (count))

primitive prim_primEnumReset :: ForeignObj -> IO (Int,Addr)
primEnumReset :: IUnknown a -> IO ()
primEnumReset gc_arg1 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_primEnumReset arg1 >>= \ (gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (return (()))

primitive prim_primEnumSkip :: ForeignObj -> Int -> IO (Int,Addr)
primEnumSkip :: IUnknown a -> Int -> IO ()
primEnumSkip gc_arg1 arg2 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_primEnumSkip arg1 arg2 >>= \ (gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (return (()))

-----------------------------------------------------------
-- Monikers & HTML Dialogs
-----------------------------------------------------------

--%fun createURLMoniker :: IMoniker a -> Pointer WideString -> IO (IMoniker Interface)
--%call (iunknown arg1) (pointer arg2)
--%code IMoniker* res1 = NULL; HRESULT hr = CreateURLMoniker( arg1,arg2,&res1);
--%fail {FAILED(hr)} {primStringFromHR(hr)}
--%result (iunknown res1)

primitive prim_createPointerMoniker :: ForeignObj -> IO (Addr,Addr,Int,Addr)
createPointerMoniker :: IUnknown a -> IO (IMoniker Interface)
createPointerMoniker gc_arg1 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  prim_createPointerMoniker arg1 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_createItemMoniker :: ForeignObj -> ForeignObj -> IO (Addr,Addr,Int,Addr)
createItemMoniker :: Pointer WideString -> Pointer WideString -> IO (IMoniker Interface)
createItemMoniker gc_arg1 gc_arg2 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg2) ->
  prim_createItemMoniker arg1 arg2 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_createFileMoniker :: ForeignObj -> IO (Addr,Addr,Int,Addr)
createFileMoniker :: Pointer WideString -> IO (IMoniker Interface)
createFileMoniker gc_arg1 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  prim_createFileMoniker arg1 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_createBindCtx :: IO (Addr,Addr,Int,Addr)
createBindCtx :: IO (IBindCtx Interface)
createBindCtx =
  prim_createBindCtx >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primMkParseDisplayName :: ForeignObj -> ForeignObj -> IO (Addr,Addr,Int,Addr)
primMkParseDisplayName :: IBindCtx a -> Pointer WideString -> IO (IMoniker Interface)
primMkParseDisplayName gc_arg1 gc_arg3 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (arg1) ->
  (marshall_pointer_ gc_arg3) >>= \ (arg2) ->
  prim_primMkParseDisplayName arg1 arg2 >>= \ (gc_res5,gc_res3,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res3 gc_res5) >>= \ gc_res4 ->
       (unmarshall_pointer_ gc_res4) >>= \ gc_res2 ->
       (unmarshall_iunknown_ gc_res2) >>= \ gc_res1 ->
       (return (gc_res1))


primitive prim_primShowHTMLDialog :: Int -> ForeignObj -> ForeignObj -> ForeignObj -> ForeignObj -> IO (Int,Addr)
primShowHTMLDialog :: Int -> IMoniker a -> Pointer variant -> Pointer String -> Pointer variant -> IO ()
primShowHTMLDialog hwnd gc_arg1 gc_arg3 gc_arg4 gc_arg5 =
  (marshall_iunknown_ gc_arg1) >>= \ (gc_arg2) ->
  (marshall_pointer_ gc_arg2) >>= \ (mk) ->
  (marshall_pointer_ gc_arg3) >>= \ (args) ->
  (marshall_pointer_ gc_arg4) >>= \ (options) ->
  (marshall_pointer_ gc_arg5) >>= \ (res) ->
  prim_primShowHTMLDialog hwnd mk args options res >>= \ (gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (return (()))

needPrims_hugs 2
