Copyright (c) 1998, Daan Leijen

> module Literal where

> data Section a        = Normal a
>                       | Section a
>                       | Start
>                       | End

> literal :: String -> [String]
> textLine :: String -> String
>
> literal       = sectionWith outLiteral . map prefix . lines
> textLine      = sectionWith outVerb . verbLine


Literal lines.

> outLiteral (Normal line) ls   = textLine line : ls
> outLiteral (Section line) ls  = line : ls
> outLiteral (Start) ls         = "{\\small\\begin{verbatim}" : ls
> outLiteral (End) ls           = "\\end{verbatim}}" : ls

> prefix ('>':s)        = (True,' ':s)
> prefix ('@':'>':s)    = (True,'.':' ':s)
> prefix s              = (False,s)


Verbatim within text lines.

> outVerb (Normal c) cs         = c:cs
> outVerb (Section c) cs        = c:cs
> outVerb (Start) cs            = "\\verb#" ++ cs
> outVerb (End) cs              = '#':cs

> verbLine                      = foldVerb False
>
> foldVerb verb ('@':'@':xs)    = (verb,'@') : foldVerb verb xs
> foldVerb verb ('@':xs)        = foldVerb (not verb) xs
> foldVerb verb (x:xs)          = (verb,x) : foldVerb verb xs
> foldVerb verb []              = []

Generic routines on sections.

> sectionWith :: (Section a -> [a] -> [a]) -> [(Bool,a)] -> [a]
> sectionWith outsection 
>               = foldr outsection [] . foldr change [] . prevsec
>               where
>                 prevsec xs  = let (is,ys)  = unzip xs
>                               in  zip3 (False : is) is ys

> change (prevInSec,inSec,x) xs
>       = prelude ( kind x : postlude xs)
>       where
>         kind x   | inSec     = Section x
>                  | otherwise = Normal x
>
>         prelude  | prevInSec && not inSec  = (End :)
>                  | not prevInSec && inSec  = (Start :)
>                  | otherwise               = id
>
>         postlude []   | inSec   = [End]
>         postlude xs             = xs


> test1  = "ho@i \n\n> code\n \n literal @co@@de@ text\n>code\n\n>code3\n\ntext@@fwi\n"
> test1inf  = concat (repeat test1)
