\documentclass{article}
\include{/hugs/lib/activex/lhs}

\begin{document}

\begin{notice}
Copyright (c) 1998,  Daan Leijen, leijen@@fwi.uva.nl

This module is part of HaskellDirect (H/Direct).
See @notice.txt@ for details and conditions of use.
\end{notice}

> module Marshall (
>                   WideString,
>                   stackString, stackWideString,
>                   stackStringLen, stackWideStringLen,
>                   writeStringAt, readStringAt,
>                   lenString,
>                   lenWideFromString, lenStringFromWide,
>                   wideFromString, stringFromWide
>                 ) where

> import Pointer ( Pointer,
>                  stackFrame, pointerInc,
>                  writeSeqAt, readSeqAt )

> import MarshallPrim (WideString, sizeWide,
>                      writeCharAt,readCharAt,readCharAtOffset,
>                      lenString,
>                      lenStringFromWide, lenWideFromString,
>                      stringFromWide, wideFromString )


I just defined marshalling functions for strings. Never
needed anything else until now :-)

A Wide (UNICODE) string just exists outside Haskell, it is
therefore an abstract data type defined in @MarshallPrim@

The @stack@String functions allocate strings in a stack like
fashion; usefull for @in@ arguments. The functions seem a
bit randomly chosen though :-(

> stackString
>       = stackStringLen 0
>
> stackStringLen :: Int -> String -> (Pointer String -> IO a) -> IO a
> stackStringLen len str f
>       = let slen = length str + 1 `max` len
>         in stackFrame slen $ \pstr ->
>            do writeStringAt str pstr
>               f pstr

> stackWideString str f
>       = stackWideStringLen 0 str (\ps pw -> f pw)
>
> stackWideStringLen :: Int -> String ->
>                        (Pointer String -> Pointer WideString -> IO a) -> IO a
> stackWideStringLen len str f
>       = stackString str $ \pstr ->
>         let wlen = lenWideFromString pstr `max` len
>         in  stackFrame (sizeWide*(wlen+1)) $ \pwide ->
>             do wideFromString pstr wlen pwide
>                f pstr pwide


Basic marshalling functions for strings.

> sizeChar      = 1
>
> writeStringAt :: String -> Pointer String -> IO ()
> writeStringAt s p
>       = writeSeqAt sizeChar (map writeCharAt (s++"\0")) p
>
> readStringAt :: Pointer String -> IO String
> readStringAt p
>       = readStr "" (lenString p -1)
>       where
>         readStr s ofs  | ofs < 0     = return s
>                        | otherwise   = do c <- readCharAtOffset p ofs
>                                           readStr (c:s) (ofs-1)


\end{document}

