-----------------------------------------------------------
-- (c) 1998,  Daan Leijen, leijen@@fwi.uva.nl
-----------------------------------------------------------

module PointerPrim where

import StdDIS


data Ptr           = Ptr ForeignObj Ptr
newtype Pointer a  = Pointer Ptr



marshall_pointer_ (Pointer (Ptr f root)) = return f
unmarshall_pointer_ f                    = let p = Ptr f p  in return (Pointer p)


primitive prim_primPointerCheck :: IO ()
primPointerCheck :: IO ()
primPointerCheck =
  prim_primPointerCheck

primitive prim_unsafeAddrFromPointer :: ForeignObj -> IO (Addr)
unsafeAddrFromPointer :: Pointer a -> IO Addr
unsafeAddrFromPointer gc_arg1 =
  (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
  prim_unsafeAddrFromPointer arg1 >>= \ (res1) ->
  (return (res1))

primitive prim_finalFreeMemory :: IO (Addr)
finalFreeMemory :: Addr
finalFreeMemory =
  unsafePerformIO(
    prim_finalFreeMemory >>= \ (res1) ->
    (return (res1)))

primitive prim_finalFreeObject :: IO (Addr)
finalFreeObject :: Addr
finalFreeObject =
  unsafePerformIO(
    prim_finalFreeObject >>= \ (res1) ->
    (return (res1)))

primitive prim_finalFreeBSTR :: IO (Addr)
finalFreeBSTR :: Addr
finalFreeBSTR =
  unsafePerformIO(
    prim_finalFreeBSTR >>= \ (res1) ->
    (return (res1)))

primitive prim_finalNoFree :: IO (Addr)
finalNoFree :: Addr
finalNoFree =
  unsafePerformIO(
    prim_finalNoFree >>= \ (res1) ->
    (return (res1)))

primitive prim_addrNULL :: IO (Addr)
addrNULL :: Addr
addrNULL =
  unsafePerformIO(
    prim_addrNULL >>= \ (res1) ->
    (return (res1)))

primitive prim_pointerNULL :: IO (Addr,Addr)
pointerNULL :: Pointer a
pointerNULL =
  unsafePerformIO(
    prim_pointerNULL >>= \ (gc_res4,gc_res2) ->
    (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
    (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
    (return (gc_res1)))

primitive prim_pointerEqual :: ForeignObj -> ForeignObj -> IO (Int)
pointerEqual :: Pointer a -> Pointer b -> Bool
pointerEqual gc_arg1 gc_arg2 =
  unsafePerformIO(
    (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
    (marshall_pointer_ gc_arg2) >>= \ (arg2) ->
    prim_pointerEqual arg1 arg2 >>= \ (res1) ->
    (unmarshall_bool_ res1) >>= \ gc_res1 ->
    (return (gc_res1)))

primitive prim_primPointerPlus :: ForeignObj -> Int -> IO (Addr,Addr)
primPointerPlus :: Pointer a -> Int -> Pointer a
primPointerPlus gc_arg1 arg2 =
  unsafePerformIO(
    (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
    prim_primPointerPlus arg1 arg2 >>= \ (gc_res4,gc_res2) ->
    (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
    (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
    (return (gc_res1)))

primitive prim_allocMemory :: Int -> IO (Addr,Addr,Int,Addr)
allocMemory :: Int -> IO (Pointer a)
allocMemory arg1 =
  prim_allocMemory arg1 >>= \ (gc_res4,gc_res2,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
       (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_allocNoFreeMemory :: Int -> IO (Addr,Addr,Int,Addr)
allocNoFreeMemory :: Int -> IO (Pointer a)
allocNoFreeMemory arg1 =
  prim_allocNoFreeMemory arg1 >>= \ (gc_res4,gc_res2,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
       (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_allocFrame :: Int -> IO (Addr,Addr,Int,Addr)
allocFrame :: Int -> IO (Pointer a)
allocFrame arg1 =
  prim_allocFrame arg1 >>= \ (gc_res4,gc_res2,gc_failed,gc_failstring) ->
  if ( gc_failed /= 0)
  then unmarshall_string_ gc_failstring >>= fail . userError
  else (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
       (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
       (return (gc_res1))

primitive prim_primFinalize :: Addr -> Addr -> IO ()
primFinalize :: Addr -> Addr -> IO ()
primFinalize arg1 arg2 =
  prim_primFinalize arg1 arg2

primitive prim_addrFromInt :: Int -> IO (Addr)
addrFromInt :: Int -> Addr
addrFromInt arg1 =
  unsafePerformIO(
    prim_addrFromInt arg1 >>= \ (res1) ->
    (return (res1)))

primitive prim_intFromAddr :: Addr -> IO (Int)
intFromAddr :: Addr -> Int
intFromAddr arg1 =
  unsafePerformIO(
    prim_intFromAddr arg1 >>= \ (res1) ->
    (return (res1)))

primitive prim_unsafePointerFromInt :: Int -> IO (Addr,Addr)
unsafePointerFromInt :: Int -> Pointer a
unsafePointerFromInt arg1 =
  unsafePerformIO(
    prim_unsafePointerFromInt arg1 >>= \ (gc_res4,gc_res2) ->
    (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
    (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
    (return (gc_res1)))

primitive prim_unsafeIntFromPointer :: ForeignObj -> IO (Int)
unsafeIntFromPointer :: Pointer a -> Int
unsafeIntFromPointer gc_arg1 =
  unsafePerformIO(
    (marshall_pointer_ gc_arg1) >>= \ (arg1) ->
    prim_unsafeIntFromPointer arg1 >>= \ (res1) ->
    (return (res1)))

primitive prim_unsafeInterfacePointerFromInt :: Int -> IO (Addr,Addr)
unsafeInterfacePointerFromInt :: Int -> IO (Pointer a)
unsafeInterfacePointerFromInt arg1 =
  prim_unsafeInterfacePointerFromInt arg1 >>= \ (gc_res4,gc_res2) ->
  (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
  (unmarshall_pointer_ gc_res3) >>= \ gc_res1 ->
  (return (gc_res1))

needPrims_hugs 2
