\documentclass{article}
\include{lhs}

\begin{document}

\begin{notice}
Copyright (c) 1998,  Daan Leijen, leijen@@fwi.uva.nl

This module is part of HaskellScript (HScript).
See @notice.txt@ for details and conditions of use.
\end{notice}

> module ScriptExtern ( extern, externRoot, externObject ) where

> import PointerPrim      (unsafePointerFromInt,unsafeIntFromPointer)
> import ComPrim          (unsafeMkInterface,unsafeGetInterfacePointer)
> import IOExts           (unsafePerformIO)

> import Com              (IUnknown, coPerformIO, coFail,putMessage)
> import Automation       (IDispatch)
> import LibRef           (Ref,newRef,updateRef,getRef,setRef)

Access to external items.
@externs@ is a 'global' list of external items.
@addExtern@ is called by the COM server to add items.
@extern@ and @externRoot@ return external items.

> externs :: Ref [(String,IDispatch a)]
> externs
>       = unsafePerformIO (newRef [])
>
> addExtern :: String -> Int -> IO ()
> addExtern name addr
>       = do let obj = externObject addr
>            --putMessage ("add object " ++ name)
>            updateRef externs (replaceItem obj)
>       where
>         replaceItem obj []         = [(name, obj)]
>         replaceItem obj (x:xs)
>               | fst x == name  = (name, obj):xs
>               | otherwise      = x : replaceItem obj xs
>
> extern :: String -> IO (IDispatch a)
> extern name
>       = do is <- getRef externs
>            case (lookup name is) of
>              Just disp   -> return disp
>              Nothing     -> coFail ("extern '" ++ (show name) ++ "': unknown external item")
>
> externRoot :: IO (IDispatch a)
> externRoot
>       = do is <- getRef externs
>            case is of
>              []               -> coFail ("externRoot: no root object available")
>              ((name,root):xs) -> return root


Hooks for the hugs server.
Don't edit the (strange, inconsistent) names !
Don't edit anything about these functions

Create a com object at the integer address.
Don't try this at home :-)

> externObject :: Int -> IDispatch a
> externObject    = unsafeMkInterface . unsafePointerFromInt

> unknownFromAddr :: Int -> IUnknown a
> unknownFromAddr = unsafeMkInterface . unsafePointerFromInt
>
> addrFromUnknown :: IUnknown a -> Int
> addrFromUnknown = unsafeIntFromPointer . unsafeGetInterfacePointer
>
> showIOError :: IO a -> IO a
> showIOError       = coPerformIO 

\end{document}

