module WShost where

import qualified Prelude
import qualified Automation (IDispatch, Interface, outIDispatch, 
                             propertyGet, inVariant, outInt, inString, function1, 
                             outString, outVariant, method0, outIUnknown, propertySet, inInt)
import qualified Com (IID, mkIID, IUnknown, CLSID, mkCLSID)

-- --------------------------------------------------
-- 
-- interface IWshShell
-- 
-- --------------------------------------------------
data WshShell a = WshShell
                   
type IWshShell a = Automation.IDispatch (WshShell a)
iidWshShell :: Com.IID (IWshShell Automation.Interface)
iidWshShell = Com.mkIID "{F935DC21-1CF0-11D0-ADB9-00C04FD58A0B}"
getSpecialFolders :: IWshShell a -> Prelude.IO (IWshCollection Automation.Interface)
getSpecialFolders =
  Automation.propertyGet "SpecialFolders" [] Automation.outIDispatch
getEnvironment :: (Automation.Variant a0) => a0 -> IWshShell a -> Prelude.IO (IWshEnvironment Automation.Interface)
getEnvironment pvarType =
  Automation.propertyGet "Environment" [Automation.inVariant pvarType] Automation.outIDispatch
run :: (Automation.Variant a0, Automation.Variant a1) => Prelude.String -> a0 -> a1 -> IWshShell a -> Prelude.IO (Prelude.Int)
run bstrCommand pvarWindowStyle pvarWaitOnReturn =
  Automation.function1 "Run" [Automation.inString bstrCommand, Automation.inVariant pvarWindowStyle, Automation.inVariant pvarWaitOnReturn] Automation.outInt
popup :: (Automation.Variant a0, Automation.Variant a1, Automation.Variant a2) => Prelude.String -> a0 -> a1 -> a2 -> IWshShell a -> Prelude.IO (Prelude.Int)
popup bstrText pvarSecondsToWait pvarTitle pvarType =
  Automation.function1 "Popup" [Automation.inString bstrText, Automation.inVariant pvarSecondsToWait, Automation.inVariant pvarTitle, Automation.inVariant pvarType] Automation.outInt
createShortcut :: Prelude.String -> IWshShell a -> Prelude.IO (Automation.IDispatch Automation.Interface)
createShortcut bstrPathLink =
  Automation.function1 "CreateShortcut" [Automation.inString bstrPathLink] Automation.outIDispatch
expandEnvironmentStrings :: Prelude.String -> IWshShell a -> Prelude.IO (Prelude.String)
expandEnvironmentStrings bstrSrc =
  Automation.function1 "ExpandEnvironmentStrings" [Automation.inString bstrSrc] Automation.outString
regRead :: (Automation.Variant a0) => Prelude.String -> IWshShell a -> Prelude.IO (a0)
regRead bstrName =
  Automation.function1 "RegRead" [Automation.inString bstrName] Automation.outVariant
regWrite :: (Automation.Variant a0, Automation.Variant a1) => Prelude.String -> a0 -> a1 -> IWshShell a -> Prelude.IO ()
regWrite bstrName pvValue pvarType =
  Automation.method0 "RegWrite" [Automation.inString bstrName, Automation.inVariant pvValue, Automation.inVariant pvarType]
regDelete :: Prelude.String -> IWshShell a -> Prelude.IO ()
regDelete bstrName =
  Automation.method0 "RegDelete" [Automation.inString bstrName]
-- --------------------------------------------------
-- 
-- interface IWshCollection
-- 
-- --------------------------------------------------
data WshCollection a = WshCollection
                        
type IWshCollection a = Automation.IDispatch (WshCollection a)
iidWshCollection :: Com.IID (IWshCollection Automation.Interface)
iidWshCollection =
  Com.mkIID "{F935DC27-1CF0-11D0-ADB9-00C04FD58A0B}"
wshItem :: (Automation.Variant a0,Automation.Variant a1) => a0 -> IWshCollection a -> Prelude.IO (a1)
wshItem pvarIndex =
  Automation.function1 "Item" [Automation.inVariant pvarIndex] Automation.outVariant
wshCount :: Automation.IDispatch a -> Prelude.IO (Prelude.Int)
wshCount = Automation.function1 "Count" [] Automation.outInt
wshGetLength :: Automation.IDispatch a -> Prelude.IO (Prelude.Int)
wshGetLength = Automation.propertyGet "length" [] Automation.outInt
wshNewEnum :: Automation.IDispatch a -> Prelude.IO (Com.IUnknown Automation.Interface)
wshNewEnum = Automation.function1 "_NewEnum" [] Automation.outIUnknown
-- --------------------------------------------------
-- 
-- interface IWshEnvironment
-- 
-- --------------------------------------------------
data WshEnvironment a = WshEnvironment
                         
type IWshEnvironment a = Automation.IDispatch (WshEnvironment a)
iidWshEnvironment :: Com.IID (IWshEnvironment Automation.Interface)
iidWshEnvironment =
  Com.mkIID "{F935DC29-1CF0-11D0-ADB9-00C04FD58A0B}"
getEnvItem :: Prelude.String -> IWshEnvironment a -> Prelude.IO (Prelude.String)
getEnvItem bstrName =
  Automation.propertyGet "Item" [Automation.inString bstrName] Automation.outString
setEnvItem :: Prelude.String -> Prelude.String -> IWshEnvironment a -> Prelude.IO ()
setEnvItem bstrName out_bstrValue =
  Automation.propertySet "Item" [Automation.inString bstrName, Automation.inString out_bstrValue]
envRemove :: Prelude.String -> IWshEnvironment a -> Prelude.IO ()
envRemove bstrName =
  Automation.method0 "Remove" [Automation.inString bstrName]
-- --------------------------------------------------
-- 
-- interface IWshShortcut
-- 
-- --------------------------------------------------
data WshShortcut a = WshShortcut
                      
type IWshShortcut a = Automation.IDispatch (WshShortcut a)
iidWshShortcut :: Com.IID (IWshShortcut Automation.Interface)
iidWshShortcut = Com.mkIID "{F935DC23-1CF0-11D0-ADB9-00C04FD58A0B}"
getFullName :: Automation.IDispatch a -> Prelude.IO (Prelude.String)
getFullName =
  Automation.propertyGet "FullName" [] Automation.outString
getArguments :: IWshShortcut a -> Prelude.IO (Prelude.String)
getArguments =
  Automation.propertyGet "Arguments" [] Automation.outString
setArguments :: Prelude.String -> IWshShortcut a -> Prelude.IO ()
setArguments out_bstrArguments =
  Automation.propertySet "Arguments" [Automation.inString out_bstrArguments]
getDescription :: IWshShortcut a -> Prelude.IO (Prelude.String)
getDescription =
  Automation.propertyGet "Description" [] Automation.outString
setDescription :: Prelude.String -> IWshShortcut a -> Prelude.IO ()
setDescription out_bstrDescription =
  Automation.propertySet "Description" [Automation.inString out_bstrDescription]
getHotkey :: IWshShortcut a -> Prelude.IO (Prelude.String)
getHotkey = Automation.propertyGet "Hotkey" [] Automation.outString
setHotkey :: Prelude.String -> IWshShortcut a -> Prelude.IO ()
setHotkey out_bstrHotKey =
  Automation.propertySet "Hotkey" [Automation.inString out_bstrHotKey]
getIconLocation :: IWshShortcut a -> Prelude.IO (Prelude.String)
getIconLocation =
  Automation.propertyGet "IconLocation" [] Automation.outString
setIconLocation :: Prelude.String -> IWshShortcut a -> Prelude.IO ()
setIconLocation out_bstrIconPath =
  Automation.propertySet "IconLocation" [Automation.inString out_bstrIconPath]
setRelativePath :: Prelude.String -> IWshShortcut a -> Prelude.IO ()
setRelativePath rhs =
  Automation.propertySet "RelativePath" [Automation.inString rhs]
getTargetPath :: Automation.IDispatch a -> Prelude.IO (Prelude.String)
getTargetPath =
  Automation.propertyGet "TargetPath" [] Automation.outString
setTargetPath :: Prelude.String -> Automation.IDispatch a -> Prelude.IO ()
setTargetPath out_bstrPath =
  Automation.propertySet "TargetPath" [Automation.inString out_bstrPath]
getWindowStyle :: IWshShortcut a -> Prelude.IO (Prelude.Int)
getWindowStyle =
  Automation.propertyGet "WindowStyle" [] Automation.outInt
setWindowStyle :: Prelude.Int -> IWshShortcut a -> Prelude.IO ()
setWindowStyle out_nShowCmd =
  Automation.propertySet "WindowStyle" [Automation.inInt out_nShowCmd]
getWorkingDirectory :: IWshShortcut a -> Prelude.IO (Prelude.String)
getWorkingDirectory =
  Automation.propertyGet "WorkingDirectory" [] Automation.outString
setWorkingDirectory :: Prelude.String -> IWshShortcut a -> Prelude.IO ()
setWorkingDirectory out_bstrWorkingDirectory =
  Automation.propertySet "WorkingDirectory" [Automation.inString out_bstrWorkingDirectory]
load :: Prelude.String -> Automation.IDispatch a -> Prelude.IO ()
load bstrPathLink =
  Automation.method0 "Load" [Automation.inString bstrPathLink]
save :: Automation.IDispatch a -> Prelude.IO ()
save = Automation.method0 "Save" []
-- --------------------------------------------------
-- 
-- interface IWshNetwork
-- 
-- --------------------------------------------------
data WshNetwork a = WshNetwork
                     
type IWshNetwork a = Automation.IDispatch (WshNetwork a)
iidWshNetwork :: Com.IID (IWshNetwork Automation.Interface)
iidWshNetwork = Com.mkIID "{F935DC25-1CF0-11D0-ADB9-00C04FD58A0B}"
getUserDomain :: IWshNetwork a -> Prelude.IO (Prelude.String)
getUserDomain =
  Automation.propertyGet "UserDomain" [] Automation.outString
getUserName :: IWshNetwork a -> Prelude.IO (Prelude.String)
getUserName =
  Automation.propertyGet "UserName" [] Automation.outString
getUserProfile :: IWshNetwork a -> Prelude.IO (Prelude.String)
getUserProfile =
  Automation.propertyGet "UserProfile" [] Automation.outString
getComputerName :: IWshNetwork a -> Prelude.IO (Prelude.String)
getComputerName =
  Automation.propertyGet "ComputerName" [] Automation.outString
getOrganization :: IWshNetwork a -> Prelude.IO (Prelude.String)
getOrganization =
  Automation.propertyGet "Organization" [] Automation.outString
getSite :: IWshNetwork a -> Prelude.IO (Prelude.String)
getSite = Automation.propertyGet "Site" [] Automation.outString
mapNetworkDrive :: (Automation.Variant a0, Automation.Variant a1, Automation.Variant a2) => Prelude.String -> Prelude.String -> a0 -> a1 -> a2 -> IWshNetwork a -> Prelude.IO ()
mapNetworkDrive bstrLocalName bstrRemoteName pvarUpdateProfile pvarUserName pvarPassword =
  Automation.method0 "MapNetworkDrive" [Automation.inString bstrLocalName, Automation.inString bstrRemoteName, Automation.inVariant pvarUpdateProfile, Automation.inVariant pvarUserName, Automation.inVariant pvarPassword]
removeNetworkDrive :: (Automation.Variant a0, Automation.Variant a1) => Prelude.String -> a0 -> a1 -> IWshNetwork a -> Prelude.IO ()
removeNetworkDrive bstrName pvarForce pvarUpdateProfile =
  Automation.method0 "RemoveNetworkDrive" [Automation.inString bstrName, Automation.inVariant pvarForce, Automation.inVariant pvarUpdateProfile]
enumNetworkDrives :: IWshNetwork a -> Prelude.IO (IWshCollection Automation.Interface)
enumNetworkDrives =
  Automation.function1 "EnumNetworkDrives" [] Automation.outIDispatch
addPrinterConnection :: (Automation.Variant a0, Automation.Variant a1, Automation.Variant a2) => Prelude.String -> Prelude.String -> a0 -> a1 -> a2 -> IWshNetwork a -> Prelude.IO ()
addPrinterConnection bstrLocalName bstrRemoteName pvarUpdateProfile pvarUserName pvarPassword =
  Automation.method0 "AddPrinterConnection" [Automation.inString bstrLocalName, Automation.inString bstrRemoteName, Automation.inVariant pvarUpdateProfile, Automation.inVariant pvarUserName, Automation.inVariant pvarPassword]
removePrinterConnection :: (Automation.Variant a0, Automation.Variant a1) => Prelude.String -> a0 -> a1 -> IWshNetwork a -> Prelude.IO ()
removePrinterConnection bstrName pvarForce pvarUpdateProfile =
  Automation.method0 "RemovePrinterConnection" [Automation.inString bstrName, Automation.inVariant pvarForce, Automation.inVariant pvarUpdateProfile]
enumPrinterConnections :: IWshNetwork a -> Prelude.IO (IWshCollection Automation.Interface)
enumPrinterConnections =
  Automation.function1 "EnumPrinterConnections" [] Automation.outIDispatch
setDefaultPrinter :: Prelude.String -> IWshNetwork a -> Prelude.IO ()
setDefaultPrinter bstrName =
  Automation.method0 "SetDefaultPrinter" [Automation.inString bstrName]
-- --------------------------------------------------
-- 
-- interface IWshURLShortcut
-- 
-- --------------------------------------------------
data WshURLShortcut a = WshURLShortcut
                         
type IWshURLShortcut a = Automation.IDispatch (WshURLShortcut a)
iidWshURLShortcut :: Com.IID (IWshURLShortcut Automation.Interface)
iidWshURLShortcut =
  Com.mkIID "{F935DC2B-1CF0-11D0-ADB9-00C04FD58A0B}"
data WshWindowStyle
 = WshHide
 | WshNormalFocus
 | WshMinimizedFocus
 | WshMaximizedFocus
 | WshNormalNoFocus
 | WshInvalid5
 | WshMinimizedNoFocus
 deriving (Prelude.Show,Prelude.Enum)

-- --------------------------------------------------
-- 
-- coclass IWshShell_Class
-- 
-- --------------------------------------------------
clsidWshShell_Class :: Com.CLSID
clsidWshShell_Class =
  Com.mkCLSID "{F935DC22-1CF0-11D0-ADB9-00C04FD58A0B}"
-- --------------------------------------------------
-- 
-- coclass IWshShortcut_Class
-- 
-- --------------------------------------------------
clsidWshShortcut_Class :: Com.CLSID
clsidWshShortcut_Class =
  Com.mkCLSID "{F935DC24-1CF0-11D0-ADB9-00C04FD58A0B}"
-- --------------------------------------------------
-- 
-- coclass IWshNetwork_Class
-- 
-- --------------------------------------------------
clsidWshNetwork_Class :: Com.CLSID
clsidWshNetwork_Class =
  Com.mkCLSID "{F935DC26-1CF0-11D0-ADB9-00C04FD58A0B}"
-- --------------------------------------------------
-- 
-- coclass IWshCollection_Class
-- 
-- --------------------------------------------------
clsidWshCollection_Class :: Com.CLSID
clsidWshCollection_Class =
  Com.mkCLSID "{F935DC28-1CF0-11D0-ADB9-00C04FD58A0B}"
-- --------------------------------------------------
-- 
-- coclass IWshEnvironment_Class
-- 
-- --------------------------------------------------
clsidWshEnvironment_Class :: Com.CLSID
clsidWshEnvironment_Class =
  Com.mkCLSID "{F935DC2A-1CF0-11D0-ADB9-00C04FD58A0B}"
-- --------------------------------------------------
-- 
-- coclass IWshURLShortcut_Class
-- 
-- --------------------------------------------------
clsidWshURLShortcut_Class :: Com.CLSID
clsidWshURLShortcut_Class =
  Com.mkCLSID "{F935DC2C-1CF0-11D0-ADB9-00C04FD58A0B}"
